from collections.abc import MutableMapping
from logging import Logger
from typing import Any, Final, overload

log: Logger
SERVICE_NAME: Final = "ec2"
ORIGIN: Final = "AWS::EC2::Instance"
IMDS_URL: Final = "http://169.254.169.254/latest/"

def initialize() -> None: ...
def get_token() -> str | None: ...
def get_metadata(token: str | None = None) -> dict[str, Any]: ...  # result of parse_metadata_json()
def parse_metadata_json(json_str: str | bytes | bytearray) -> dict[str, Any]: ...  # result of json.loads()
@overload
def do_request(url: str, headers: MutableMapping[str, str] | None = None, method: str = "GET") -> str: ...
@overload
def do_request(url: None, headers: MutableMapping[str, str] | None = None, method: str = "GET") -> None: ...



from _typeshed import Incomplete
from collections.abc import Callable
from typing import Any, Literal, overload

import numpy as np
from numpy.typing import ArrayLike, NDArray

BFIndex: Incomplete

class Index:
    ef: int
    num_threads: int

    @overload
    def __init__(self, params: dict[str, Any]) -> None: ...
    @overload
    def __init__(self, index: Index) -> None: ...
    @overload
    def __init__(self, space: Literal["l2", "ip", "cosine"], dim: int) -> None: ...
    def add_items(
        self, data: ArrayLike, ids: ArrayLike | None = None, num_threads: int = -1, replace_deleted: bool = False
    ) -> None: ...
    def get_current_count(self) -> int: ...
    def get_ids_list(self) -> list[int]: ...
    @overload
    def get_items(self, ids: ArrayLike | None = ..., return_type: Literal["list"] = ...) -> list[float]: ...
    @overload
    def get_items(self, ids: ArrayLike | None = ..., return_type: Literal["numpy"] = ...) -> NDArray[np.float32]: ...
    @overload
    def get_items(
        self, ids: ArrayLike | None = None, return_type: Literal["numpy", "list"] = "numpy"
    ) -> NDArray[np.float32] | list[float]: ...
    def get_max_elements(self) -> int: ...
    def index_file_size(self) -> int: ...
    def init_index(
        self,
        max_elements: int,
        M: int = 16,
        ef_construction: int = 200,
        random_seed: int = 100,
        allow_replace_delete: bool = False,
    ) -> None: ...
    def knn_query(
        self, data: ArrayLike, k: int = 1, num_threads: int = -1, filter: Callable[[int], bool] | None = None
    ) -> tuple[NDArray[np.uint64], NDArray[np.float32]]: ...
    def load_index(self, path_to_index: str, max_elements: int = 0, allow_replace_delete: bool = False) -> None: ...
    def mark_deleted(self, label: int) -> None: ...
    def resize_index(self, new_size: int) -> None: ...
    def save_index(self, path_to_index: str) -> None: ...
    def set_ef(self, ef: int) -> None: ...
    def set_num_threads(self, num_threads: int) -> None: ...
    def unmark_deleted(self, label: int) -> None: ...
    @property
    def M(self) -> int: ...
    @property
    def dim(self) -> int: ...
    @property
    def ef_construction(self) -> int: ...
    @property
    def element_count(self) -> int: ...
    @property
    def max_elements(self) -> int: ...
    @property
    def space(self) -> str: ...



from _typeshed import Incomplete

from braintree.attribute_getter import AttributeGetter

class ExchangeRateQuoteInput(AttributeGetter):
    parent: Incomplete
    def __init__(self, parent, attributes) -> None: ...
    def done(self): ...
    def to_graphql_variables(self) -> dict[str, Incomplete]: ...



from .code import OpenIDCode as OpenIDCode, OpenIDToken as OpenIDToken
from .hybrid import OpenIDHybridGrant as OpenIDHybridGrant
from .implicit import OpenIDImplicitGrant as OpenIDImplicitGrant

__all__ = ["OpenIDToken", "OpenIDCode", "OpenIDImplicitGrant", "OpenIDHybridGrant"]



from decimal import Decimal

from braintree.attribute_getter import AttributeGetter

class AuthorizationAdjustment(AttributeGetter):
    amount: Decimal | None
    def __init__(self, attributes) -> None: ...



from _typeshed import Incomplete
from collections.abc import Generator, Iterable
from dataclasses import dataclass

from .fonts import TextStyle
from .fpdf import FPDF
from .structure_tree import StructElem
from .syntax import Destination, PDFObject, PDFString

@dataclass
class OutlineSection:
    __slots__ = ("name", "level", "page_number", "dest", "struct_elem")
    name: str
    level: int
    page_number: int
    dest: Destination
    struct_elem: StructElem | None = None

class OutlineItemDictionary(PDFObject):
    __slots__ = ("_id", "title", "parent", "prev", "next", "first", "last", "count", "dest", "struct_elem")
    title: PDFString
    parent: Incomplete | None
    prev: Incomplete | None
    next: Incomplete | None
    first: Incomplete | None
    last: Incomplete | None
    count: int
    dest: Destination | None
    struct_elem: StructElem | None
    def __init__(self, title: str, dest: Destination | None = None, struct_elem: StructElem | None = None) -> None: ...

class OutlineDictionary(PDFObject):
    __slots__ = ("_id", "type", "first", "last", "count")
    type: str
    first: Incomplete | None
    last: Incomplete | None
    count: int
    def __init__(self) -> None: ...

def build_outline_objs(
    sections: Iterable[Incomplete],
) -> Generator[Incomplete, None, list[OutlineDictionary | OutlineItemDictionary]]: ...

class TableOfContents:
    text_style: TextStyle
    use_section_title_styles: bool
    level_indent: float
    line_spacing: float
    ignore_pages_before_toc: bool

    def __init__(
        self,
        text_style: TextStyle | None = None,
        use_section_title_styles: bool = False,
        level_indent: float = 7.5,
        line_spacing: float = 1.5,
        ignore_pages_before_toc: bool = True,
    ) -> None: ...
    def get_text_style(self, pdf: FPDF, item: OutlineSection) -> TextStyle: ...
    def render_toc_item(self, pdf: FPDF, item: OutlineSection) -> None: ...
    def render_toc(self, pdf: FPDF, outline: Iterable[OutlineSection]) -> None: ...



from _typeshed import Incomplete
from collections.abc import Collection
from typing_extensions import TypeAlias

from authlib.oauth2 import OAuth2Request
from authlib.oauth2.rfc6749 import ClientMixin

from ..hooks import Hookable

_ServerResponse: TypeAlias = tuple[int, str, list[tuple[str, str]]]

class BaseGrant(Hookable):
    TOKEN_ENDPOINT_AUTH_METHODS: Collection[str]
    GRANT_TYPE: str | None
    TOKEN_RESPONSE_HEADER: Collection[tuple[str, str]]
    prompt: Incomplete
    redirect_uri: Incomplete
    request: OAuth2Request
    server: Incomplete
    def __init__(self, request: OAuth2Request, server) -> None: ...
    @property
    def client(self): ...
    def generate_token(
        self,
        user=None,
        scope: str | None = None,
        grant_type: str | None = None,
        expires_in: int | None = None,
        include_refresh_token: bool = True,
    ) -> dict[str, str | int]: ...
    def authenticate_token_endpoint_client(self) -> ClientMixin: ...
    def save_token(self, token): ...
    def validate_requested_scope(self) -> None: ...

class TokenEndpointMixin:
    TOKEN_ENDPOINT_HTTP_METHODS: Incomplete
    GRANT_TYPE: Incomplete
    @classmethod
    def check_token_endpoint(cls, request: OAuth2Request): ...
    def validate_token_request(self) -> None: ...
    def create_token_response(self) -> _ServerResponse: ...

class AuthorizationEndpointMixin:
    RESPONSE_TYPES: Collection[str]
    ERROR_RESPONSE_FRAGMENT: bool
    @classmethod
    def check_authorization_endpoint(cls, request: OAuth2Request) -> bool: ...
    @staticmethod
    def validate_authorization_redirect_uri(request: OAuth2Request, client: ClientMixin) -> str: ...
    @staticmethod
    def validate_no_multiple_request_parameter(request: OAuth2Request): ...
    redirect_uri: str
    def validate_consent_request(self) -> str: ...
    def validate_authorization_request(self) -> str: ...
    def create_authorization_response(self, redirect_uri: str, grant_user) -> _ServerResponse: ...



from logging import Logger

from aws_xray_sdk.core.daemon_config import DaemonConfig

log: Logger

class DefaultSampler:
    def __init__(self) -> None: ...
    def start(self) -> None: ...
    def should_trace(self, sampling_req=None): ...
    def load_local_rules(self, rules) -> None: ...
    def load_settings(self, daemon_config: DaemonConfig, context, origin=None) -> None: ...
    @property
    def xray_client(self): ...
    @xray_client.setter
    def xray_client(self, v) -> None: ...



from _typeshed import Incomplete

from .token_verifier import AsymmetricSignatureVerifier, JwksFetcher, TokenVerifier

class AsyncAsymmetricSignatureVerifier(AsymmetricSignatureVerifier):
    def __init__(self, jwks_url: str, algorithm: str = "RS256") -> None: ...
    def set_session(self, session) -> None: ...

class AsyncJwksFetcher(JwksFetcher):
    def __init__(self, *args, **kwargs) -> None: ...
    def set_session(self, session) -> None: ...
    async def get_key(self, key_id: str): ...

class AsyncTokenVerifier(TokenVerifier):
    iss: str
    aud: str
    leeway: int
    def __init__(
        self, signature_verifier: AsyncAsymmetricSignatureVerifier, issuer: str, audience: str, leeway: int = 0
    ) -> None: ...
    def set_session(self, session) -> None: ...
    async def verify(  # type: ignore[override] # Differs from supertype
        self, token: str, nonce: str | None = None, max_age: int | None = None, organization: str | None = None
    ) -> dict[str, Incomplete]: ...



from collections.abc import Generator

from networkx.classes.graph import Graph, _Node
from networkx.utils.backends import _dispatchable

__all__ = ["biconnected_components", "biconnected_component_edges", "is_biconnected", "articulation_points"]

@_dispatchable
def is_biconnected(G: Graph[_Node]) -> bool: ...
@_dispatchable
def biconnected_component_edges(G: Graph[_Node]) -> Generator[list[tuple[_Node, _Node]]]: ...
@_dispatchable
def biconnected_components(G: Graph[_Node]) -> Generator[list[set[_Node]]]: ...
@_dispatchable
def articulation_points(G: Graph[_Node]) -> Generator[_Node]: ...



"""The Jac Programming Language."""

import sys

from jaclang.meta_importer import JacMetaImporter

# Register JacMetaImporter BEFORE loading plugins, so .jac modules can be imported
if not any(isinstance(f, JacMetaImporter) for f in sys.meta_path):
    sys.meta_path.insert(0, JacMetaImporter())

# Import compiler first to ensure generated parsers exist before pycore.parser is loaded
# Backwards-compatible import path for older plugins/tests.
# Prefer `jaclang.pycore.runtime` going forward.
import jaclang.pycore.runtime as _runtime_mod
from jaclang import compiler as _compiler  # noqa: F401
from jaclang.pycore.helpers import get_disabled_plugins, load_plugins_with_disabling
from jaclang.pycore.runtime import (
    JacRuntime,
    JacRuntimeImpl,
    JacRuntimeInterface,
    plugin_manager,
)

sys.modules.setdefault("jaclang.runtimelib.runtime", _runtime_mod)

plugin_manager.register(JacRuntimeImpl)

# Load external plugins with disabling support
# Disabling can be configured via JAC_DISABLED_PLUGINS env var or jac.toml [plugins].disabled
# Use "*" to disable all external plugins, "package:*" for all from a package,
# or "package:plugin" for specific plugins
_disabled_list = get_disabled_plugins()
if _disabled_list:
    # Use qualified blocking for fine-grained control
    load_plugins_with_disabling(plugin_manager, _disabled_list)
else:
    # No disabling - load all plugins normally
    plugin_manager.load_setuptools_entrypoints("jac")

__all__ = ["JacRuntimeInterface", "JacRuntime"]



from typing import ClassVar, TypedDict, type_check_only
from typing_extensions import NotRequired

from .reservoir import Reservoir

@type_check_only
class _Rule(TypedDict):
    description: NotRequired[str]
    host: NotRequired[str]
    service_name: NotRequired[str]
    http_method: NotRequired[str]
    url_path: NotRequired[str]
    fixed_target: NotRequired[int]
    rate: NotRequired[float]

class SamplingRule:
    FIXED_TARGET: ClassVar[str]
    RATE: ClassVar[str]
    HOST: ClassVar[str]
    METHOD: ClassVar[str]
    PATH: ClassVar[str]
    SERVICE_NAME: ClassVar[str]
    def __init__(self, rule_dict: _Rule, version: int = 2, default: bool = False) -> None: ...
    def applies(self, host: str | None, method: str | None, path: str | None) -> bool: ...
    @property
    def fixed_target(self) -> int | None: ...
    @property
    def rate(self) -> float | None: ...
    @property
    def host(self) -> str | None: ...
    @property
    def method(self) -> str | None: ...
    @property
    def path(self) -> str | None: ...
    @property
    def reservoir(self) -> Reservoir: ...
    @property
    def version(self): ...



from typing import Final

from .doctemplate import *
from .flowables import *
from .frames import *
from .multicol import *
from .paragraph import *
from .paraparser import *
from .tables import *
from .xpreformatted import *

__version__: Final[str]



"""
@generated by mypy-protobuf.  Do not edit manually!
isort:skip_file
"""

import builtins
import collections.abc
import sys
import typing

import google.protobuf.descriptor
import google.protobuf.internal.containers
import google.protobuf.internal.enum_type_wrapper
import google.protobuf.message
import tensorflow.core.framework.tensor_pb2
import tensorflow.tsl.protobuf.histogram_pb2

if sys.version_info >= (3, 10):
    import typing as typing_extensions
else:
    import typing_extensions
from tensorflow.tsl.protobuf.histogram_pb2 import HistogramProto as HistogramProto

DESCRIPTOR: google.protobuf.descriptor.FileDescriptor

class _DataClass:
    ValueType = typing.NewType("ValueType", builtins.int)
    V: typing_extensions.TypeAlias = ValueType

class _DataClassEnumTypeWrapper(google.protobuf.internal.enum_type_wrapper._EnumTypeWrapper[_DataClass.ValueType], builtins.type):
    DESCRIPTOR: google.protobuf.descriptor.EnumDescriptor
    DATA_CLASS_UNKNOWN: _DataClass.ValueType  # 0
    """Unknown data class, used (implicitly) for legacy data. Will not be
    processed by data ingestion pipelines.
    """
    DATA_CLASS_SCALAR: _DataClass.ValueType  # 1
    """Scalar time series. Each `Value` for the corresponding tag must have
    `tensor` set to a rank-0 tensor of type `DT_FLOAT` (float32).
    """
    DATA_CLASS_TENSOR: _DataClass.ValueType  # 2
    """Tensor time series. Each `Value` for the corresponding tag must have
    `tensor` set. The tensor value is arbitrary, but should be small to
    accommodate direct storage in database backends: an upper bound of a few
    kilobytes is a reasonable rule of thumb.
    """
    DATA_CLASS_BLOB_SEQUENCE: _DataClass.ValueType  # 3
    """Blob sequence time series. Each `Value` for the corresponding tag must
    have `tensor` set to a rank-1 tensor of bytestring dtype.
    """

class DataClass(_DataClass, metaclass=_DataClassEnumTypeWrapper): ...

DATA_CLASS_UNKNOWN: DataClass.ValueType  # 0
"""Unknown data class, used (implicitly) for legacy data. Will not be
processed by data ingestion pipelines.
"""
DATA_CLASS_SCALAR: DataClass.ValueType  # 1
"""Scalar time series. Each `Value` for the corresponding tag must have
`tensor` set to a rank-0 tensor of type `DT_FLOAT` (float32).
"""
DATA_CLASS_TENSOR: DataClass.ValueType  # 2
"""Tensor time series. Each `Value` for the corresponding tag must have
`tensor` set. The tensor value is arbitrary, but should be small to
accommodate direct storage in database backends: an upper bound of a few
kilobytes is a reasonable rule of thumb.
"""
DATA_CLASS_BLOB_SEQUENCE: DataClass.ValueType  # 3
"""Blob sequence time series. Each `Value` for the corresponding tag must
have `tensor` set to a rank-1 tensor of bytestring dtype.
"""
global___DataClass = DataClass

@typing.final
class SummaryDescription(google.protobuf.message.Message):
    """Metadata associated with a series of Summary data"""

    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    TYPE_HINT_FIELD_NUMBER: builtins.int
    type_hint: builtins.str
    """Hint on how plugins should process the data in this series.
    Supported values include "scalar", "histogram", "image", "audio"
    """
    def __init__(self, *, type_hint: builtins.str | None = ...) -> None: ...
    def ClearField(self, field_name: typing.Literal["type_hint", b"type_hint"]) -> None: ...

global___SummaryDescription = SummaryDescription

@typing.final
class SummaryMetadata(google.protobuf.message.Message):
    """A SummaryMetadata encapsulates information on which plugins are able to make
    use of a certain summary value.
    """

    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    @typing.final
    class PluginData(google.protobuf.message.Message):
        DESCRIPTOR: google.protobuf.descriptor.Descriptor

        PLUGIN_NAME_FIELD_NUMBER: builtins.int
        CONTENT_FIELD_NUMBER: builtins.int
        plugin_name: builtins.str
        """The name of the plugin this data pertains to."""
        content: builtins.bytes
        """The content to store for the plugin. The best practice is for this to be
        a binary serialized protocol buffer.
        """
        def __init__(self, *, plugin_name: builtins.str | None = ..., content: builtins.bytes | None = ...) -> None: ...
        def ClearField(self, field_name: typing.Literal["content", b"content", "plugin_name", b"plugin_name"]) -> None: ...

    PLUGIN_DATA_FIELD_NUMBER: builtins.int
    DISPLAY_NAME_FIELD_NUMBER: builtins.int
    SUMMARY_DESCRIPTION_FIELD_NUMBER: builtins.int
    DATA_CLASS_FIELD_NUMBER: builtins.int
    display_name: builtins.str
    """Display name for viewing in TensorBoard."""
    summary_description: builtins.str
    """Longform readable description of the summary sequence. Markdown supported."""
    data_class: global___DataClass.ValueType
    """Class of data stored in this time series. Required for compatibility with
    TensorBoard's generic data facilities (`DataProvider`, et al.). This value
    imposes constraints on the dtype and shape of the corresponding tensor
    values. See `DataClass` docs for details.
    """
    @property
    def plugin_data(self) -> global___SummaryMetadata.PluginData:
        """Data that associates a summary with a certain plugin."""

    def __init__(
        self,
        *,
        plugin_data: global___SummaryMetadata.PluginData | None = ...,
        display_name: builtins.str | None = ...,
        summary_description: builtins.str | None = ...,
        data_class: global___DataClass.ValueType | None = ...,
    ) -> None: ...
    def HasField(self, field_name: typing.Literal["plugin_data", b"plugin_data"]) -> builtins.bool: ...
    def ClearField(
        self,
        field_name: typing.Literal[
            "data_class",
            b"data_class",
            "display_name",
            b"display_name",
            "plugin_data",
            b"plugin_data",
            "summary_description",
            b"summary_description",
        ],
    ) -> None: ...

global___SummaryMetadata = SummaryMetadata

@typing.final
class Summary(google.protobuf.message.Message):
    """A Summary is a set of named values to be displayed by the
    visualizer.

    Summaries are produced regularly during training, as controlled by
    the "summary_interval_secs" attribute of the training operation.
    Summaries are also produced at the end of an evaluation.
    """

    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    @typing.final
    class Image(google.protobuf.message.Message):
        DESCRIPTOR: google.protobuf.descriptor.Descriptor

        HEIGHT_FIELD_NUMBER: builtins.int
        WIDTH_FIELD_NUMBER: builtins.int
        COLORSPACE_FIELD_NUMBER: builtins.int
        ENCODED_IMAGE_STRING_FIELD_NUMBER: builtins.int
        height: builtins.int
        """Dimensions of the image."""
        width: builtins.int
        colorspace: builtins.int
        """Valid colorspace values are
          1 - grayscale
          2 - grayscale + alpha
          3 - RGB
          4 - RGBA
          5 - DIGITAL_YUV
          6 - BGRA
        """
        encoded_image_string: builtins.bytes
        """Image data in encoded format.  All image formats supported by
        image_codec::CoderUtil can be stored here.
        """
        def __init__(
            self,
            *,
            height: builtins.int | None = ...,
            width: builtins.int | None = ...,
            colorspace: builtins.int | None = ...,
            encoded_image_string: builtins.bytes | None = ...,
        ) -> None: ...
        def ClearField(
            self,
            field_name: typing.Literal[
                "colorspace",
                b"colorspace",
                "encoded_image_string",
                b"encoded_image_string",
                "height",
                b"height",
                "width",
                b"width",
            ],
        ) -> None: ...

    @typing.final
    class Audio(google.protobuf.message.Message):
        DESCRIPTOR: google.protobuf.descriptor.Descriptor

        SAMPLE_RATE_FIELD_NUMBER: builtins.int
        NUM_CHANNELS_FIELD_NUMBER: builtins.int
        LENGTH_FRAMES_FIELD_NUMBER: builtins.int
        ENCODED_AUDIO_STRING_FIELD_NUMBER: builtins.int
        CONTENT_TYPE_FIELD_NUMBER: builtins.int
        sample_rate: builtins.float
        """Sample rate of the audio in Hz."""
        num_channels: builtins.int
        """Number of channels of audio."""
        length_frames: builtins.int
        """Length of the audio in frames (samples per channel)."""
        encoded_audio_string: builtins.bytes
        """Encoded audio data and its associated RFC 2045 content type (e.g.
        "audio/wav").
        """
        content_type: builtins.str
        def __init__(
            self,
            *,
            sample_rate: builtins.float | None = ...,
            num_channels: builtins.int | None = ...,
            length_frames: builtins.int | None = ...,
            encoded_audio_string: builtins.bytes | None = ...,
            content_type: builtins.str | None = ...,
        ) -> None: ...
        def ClearField(
            self,
            field_name: typing.Literal[
                "content_type",
                b"content_type",
                "encoded_audio_string",
                b"encoded_audio_string",
                "length_frames",
                b"length_frames",
                "num_channels",
                b"num_channels",
                "sample_rate",
                b"sample_rate",
            ],
        ) -> None: ...

    @typing.final
    class Value(google.protobuf.message.Message):
        DESCRIPTOR: google.protobuf.descriptor.Descriptor

        NODE_NAME_FIELD_NUMBER: builtins.int
        TAG_FIELD_NUMBER: builtins.int
        METADATA_FIELD_NUMBER: builtins.int
        SIMPLE_VALUE_FIELD_NUMBER: builtins.int
        OBSOLETE_OLD_STYLE_HISTOGRAM_FIELD_NUMBER: builtins.int
        IMAGE_FIELD_NUMBER: builtins.int
        HISTO_FIELD_NUMBER: builtins.int
        AUDIO_FIELD_NUMBER: builtins.int
        TENSOR_FIELD_NUMBER: builtins.int
        node_name: builtins.str
        """This field is deprecated and will not be set."""
        tag: builtins.str
        """Tag name for the data. Used by TensorBoard plugins to organize data. Tags
        are often organized by scope (which contains slashes to convey
        hierarchy). For example: foo/bar/0
        """
        simple_value: builtins.float
        obsolete_old_style_histogram: builtins.bytes
        @property
        def metadata(self) -> global___SummaryMetadata:
            """Contains metadata on the summary value such as which plugins may use it.
            Take note that many summary values may lack a metadata field. This is
            because the FileWriter only keeps a metadata object on the first summary
            value with a certain tag for each tag. TensorBoard then remembers which
            tags are associated with which plugins. This saves space.
            """

        @property
        def image(self) -> global___Summary.Image: ...
        @property
        def histo(self) -> tensorflow.tsl.protobuf.histogram_pb2.HistogramProto: ...
        @property
        def audio(self) -> global___Summary.Audio: ...
        @property
        def tensor(self) -> tensorflow.core.framework.tensor_pb2.TensorProto: ...
        def __init__(
            self,
            *,
            node_name: builtins.str | None = ...,
            tag: builtins.str | None = ...,
            metadata: global___SummaryMetadata | None = ...,
            simple_value: builtins.float | None = ...,
            obsolete_old_style_histogram: builtins.bytes | None = ...,
            image: global___Summary.Image | None = ...,
            histo: tensorflow.tsl.protobuf.histogram_pb2.HistogramProto | None = ...,
            audio: global___Summary.Audio | None = ...,
            tensor: tensorflow.core.framework.tensor_pb2.TensorProto | None = ...,
        ) -> None: ...
        def HasField(
            self,
            field_name: typing.Literal[
                "audio",
                b"audio",
                "histo",
                b"histo",
                "image",
                b"image",
                "metadata",
                b"metadata",
                "obsolete_old_style_histogram",
                b"obsolete_old_style_histogram",
                "simple_value",
                b"simple_value",
                "tensor",
                b"tensor",
                "value",
                b"value",
            ],
        ) -> builtins.bool: ...
        def ClearField(
            self,
            field_name: typing.Literal[
                "audio",
                b"audio",
                "histo",
                b"histo",
                "image",
                b"image",
                "metadata",
                b"metadata",
                "node_name",
                b"node_name",
                "obsolete_old_style_histogram",
                b"obsolete_old_style_histogram",
                "simple_value",
                b"simple_value",
                "tag",
                b"tag",
                "tensor",
                b"tensor",
                "value",
                b"value",
            ],
        ) -> None: ...
        def WhichOneof(
            self, oneof_group: typing.Literal["value", b"value"]
        ) -> typing.Literal["simple_value", "obsolete_old_style_histogram", "image", "histo", "audio", "tensor"] | None: ...

    VALUE_FIELD_NUMBER: builtins.int
    @property
    def value(self) -> google.protobuf.internal.containers.RepeatedCompositeFieldContainer[global___Summary.Value]:
        """Set of values for the summary."""

    def __init__(self, *, value: collections.abc.Iterable[global___Summary.Value] | None = ...) -> None: ...
    def ClearField(self, field_name: typing.Literal["value", b"value"]) -> None: ...

global___Summary = Summary



from _typeshed import Incomplete
from collections.abc import Generator

def is_iterable(obj) -> bool: ...
def is_scalar(obj) -> bool: ...
def is_collection(obj) -> bool: ...
def split(src, sep=None, maxsplit=None): ...
def split_iter(src, sep=None, maxsplit=None) -> Generator[Incomplete, None, Incomplete]: ...
def lstrip(iterable, strip_value=None): ...
def lstrip_iter(iterable, strip_value=None) -> Generator[Incomplete, None, None]: ...
def rstrip(iterable, strip_value=None): ...
def rstrip_iter(iterable, strip_value=None) -> Generator[Incomplete, None, None]: ...
def strip(iterable, strip_value=None): ...
def strip_iter(iterable, strip_value=None): ...
def chunked(src, size, count=None, **kw): ...
def chunked_iter(src, size, **kw) -> Generator[Incomplete, None, Incomplete]: ...
def chunk_ranges(
    input_size: int, chunk_size: int, input_offset: int = 0, overlap_size: int = 0, align: bool = False
) -> Generator[tuple[int, int], None, None]: ...
def pairwise(src, end=...): ...
def pairwise_iter(src, end=...): ...
def windowed(src, size, fill=...): ...
def windowed_iter(src, size, fill=...): ...
def xfrange(stop, start=None, step: float = 1.0) -> Generator[Incomplete, None, None]: ...
def frange(stop, start=None, step: float = 1.0): ...
def backoff(start, stop, count=None, factor: float = 2.0, jitter: bool = False): ...
def backoff_iter(start, stop, count=None, factor: float = 2.0, jitter: bool = False) -> Generator[Incomplete, None, None]: ...
def bucketize(src, key=..., value_transform=None, key_filter=None): ...
def partition(src, key=...): ...
def unique(src, key=None): ...
def unique_iter(src, key=None) -> Generator[Incomplete, None, Incomplete]: ...
def redundant(src, key=None, groups: bool = False): ...
def one(src, default=None, key=None): ...
def first(iterable, default=None, key=None): ...
def flatten_iter(iterable) -> Generator[Incomplete, None, None]: ...
def flatten(iterable): ...
def same(iterable, ref=...): ...
def default_visit(path, key, value): ...
def default_enter(path, key, value): ...
def default_exit(path, key, old_parent, new_parent, new_items): ...
def remap(root, visit=..., enter=..., exit=..., **kwargs): ...

class PathAccessError(KeyError, IndexError, TypeError):
    exc: Incomplete
    seg: Incomplete
    path: Incomplete
    def __init__(self, exc, seg, path) -> None: ...

def get_path(root, path, default=...): ...
def research(root, query=..., reraise: bool = False, enter=...): ...

class GUIDerator:
    size: Incomplete
    count: Incomplete
    def __init__(self, size: int = 24) -> None: ...
    pid: Incomplete
    salt: Incomplete
    def reseed(self) -> None: ...
    def __iter__(self): ...
    def __next__(self): ...
    next: Incomplete

class SequentialGUIDerator(GUIDerator):
    start: Incomplete
    def reseed(self) -> None: ...
    def __next__(self): ...
    next: Incomplete

guid_iter: Incomplete
seq_guid_iter: Incomplete

def soft_sorted(iterable, first=None, last=None, key=None, reverse: bool = False): ...
def untyped_sorted(iterable, key=None, reverse: bool = False): ...



from typing import Final, Literal

EPOCH: Final = 2375839.5
YEAR_EPOCH: Final = 1791.0
DAYS_IN_YEAR: Final = 365.0
MOIS: Final[list[str]]
MONTHS: Final[list[str]]
LEAP_CYCLE_DAYS: Final = 1461.0
LEAP_CYCLE_YEARS: Final = 4.0

def leap(year: int, method: Literal[4, 100, 128, "continuous", "madler", "romme", "equinox"] | None = None) -> bool: ...
def premier_da_la_annee(jd: float) -> float: ...
def to_jd(
    year: int, month: int, day: int, method: Literal[4, 100, 128, "continuous", "madler", "romme", "equinox"] | None = None
) -> float: ...
def from_jd(
    jd: float, method: Literal[4, 100, 128, "continuous", "madler", "romme", "equinox"] | None = None
) -> tuple[int, int, int]: ...
def decade(jour: float) -> int: ...
def day_name(month: int, day: int) -> str: ...
def from_gregorian(
    year: int, month: int, day: int, method: Literal[4, 100, 128, "continuous", "madler", "romme", "equinox"] | None = None
) -> tuple[int, int, int]: ...
def to_gregorian(
    an: int, mois: int, jour: int, method: Literal[4, 100, 128, "continuous", "madler", "romme", "equinox"] | None = None
) -> tuple[int, int, int]: ...
def format(an: int, mois: int, jour: int) -> str: ...



from .aiows import AioClient as AioClient, AioServer as AioServer
from .errors import ConnectionClosed as ConnectionClosed, ConnectionError as ConnectionError
from .ws import Client as Client, Server as Server



from collections.abc import Callable
from datetime import datetime
from typing import Any

from django.db import models
from django.db.models import Model

def deprecate(msg: str, level_modifier: int = 0) -> None: ...

class MigrationNotice(DeprecationWarning):
    url: str
    def __init__(self, message: str) -> None: ...

class RenameAttributesBase(type):
    renamed_attributes: tuple[tuple[str, str, DeprecationWarning], ...] = ()

    # Class attrs vary by definition
    def __new__(metacls, name: str, bases: tuple[type, ...], attrs: dict[str, Any]) -> RenameAttributesBase: ...
    def get_name(metacls, name: str) -> str: ...
    def __getattr__(metacls, name: str) -> Any: ...  # Attribute values vary by name and class
    def __setattr__(metacls, name: str, value: Any) -> None: ...  # Attribute values can be any type

def try_dbfield(
    fn: Callable[[models.Field[Any, Any]], Any], field_class: type[models.Field[Any, Any]]
) -> Any: ...  # Generic field operation
def get_all_model_fields(model: type[Model]) -> dict[str, models.Field[Any, Any]]: ...  # Fields vary by model definition
def get_model_field(model: type[Model], field_name: str) -> models.Field[Any, Any]: ...  # Field type unknown at static time
def get_field_parts(model: type[Model], field_name: str) -> list[models.Field[Any, Any]]: ...  # Relationship fields vary
def resolve_field(
    model_field: models.Field[Any, Any], lookup_expr: str
) -> tuple[models.Field[Any, Any], str]: ...  # Generic field resolution
def handle_timezone(value: datetime, is_dst: bool | None = None) -> datetime: ...
def verbose_field_name(model: type[Model], field_name: str) -> str: ...
def verbose_lookup_expr(lookup_expr: str) -> str: ...
def label_for_filter(model: type[Model], field_name: str, lookup_expr: str, exclude: bool = False) -> str: ...
def translate_validation(error_dict: dict[str, list[str]]) -> dict[str, list[str]]: ...



import threading
from collections.abc import Mapping

from ..extractor.common import _InfoDict
from ..utils.networking import HTTPHeaderDict
from .common import FileDownloader

class BunnyCdnFD(FileDownloader):
    def real_download(self, filename: str, info_dict: _InfoDict) -> bool | None: ...
    def ping_thread(
        self,
        stop_event: threading.Event,
        url: str,
        headers: HTTPHeaderDict | Mapping[str, str] | None,
        secret: str,
        context_id: str,
    ) -> None: ...



from typing_extensions import Self

unicode = str
__tracebackhide__: bool

class StringMixin:
    def is_equal_to_ignoring_case(self, other: str) -> Self: ...
    def contains_ignoring_case(self, *items: str) -> Self: ...
    def starts_with(self, prefix: str) -> Self: ...
    def ends_with(self, suffix: str) -> Self: ...
    def matches(self, pattern: str) -> Self: ...
    def does_not_match(self, pattern: str) -> Self: ...
    def is_alpha(self) -> Self: ...
    def is_digit(self) -> Self: ...
    def is_lower(self) -> Self: ...
    def is_upper(self) -> Self: ...
    def is_unicode(self) -> Self: ...



from . import (
    bdist as bdist,
    bdist_rpm as bdist_rpm,
    build as build,
    build_clib as build_clib,
    build_ext as build_ext,
    build_py as build_py,
    # build_scripts as build_scripts,
    # check as check,
    # clean as clean,
    install as install,
    install_data as install_data,
    # install_headers as install_headers,
    install_lib as install_lib,
    install_scripts as install_scripts,
    sdist as sdist,
)

# Commented out commands are not stubbed.
# (Many of these may be implementation details,
# but they can be added if people ask for them)
__all__ = [
    "build",
    "build_py",
    "build_ext",
    "build_clib",
    # "build_scripts",
    # "clean",
    "install",
    "install_lib",
    # "install_headers",
    "install_scripts",
    "install_data",
    "sdist",
    "bdist",
    # "bdist_dumb",
    "bdist_rpm",
    # "check",
]



class AtomicCounter:
    value: int
    def __init__(self, initial: int = 0) -> None: ...
    def increment(self, num: int = 1) -> int: ...
    def decrement(self, num: int = 1) -> int: ...
    def get_current(self) -> int: ...
    def reset(self) -> int: ...



import ast
import math
import re
import time
from _typeshed import Incomplete
from collections.abc import Generator
from typing import NoReturn

eval_debug: int
strTypes: tuple[type[bytes], type[str]]
isPy39: bool
isPy313: bool

class BadCode(ValueError): ...

augOps: dict[ast.operator, str]
__allowed_magic_methods__: frozenset[str]
__rl_unsafe__: frozenset[str]
__rl_unsafe_re__: re.Pattern[str]

def copy_locations(new_node, old_node) -> None: ...

class UntrustedAstTransformer(ast.NodeTransformer):
    names_seen: Incomplete
    nameIsAllowed: Incomplete
    def __init__(self, names_seen=None, nameIsAllowed=None) -> None: ...
    @property
    def tmpName(self): ...
    def error(self, node, msg) -> NoReturn: ...
    def guard_iter(self, node): ...
    def is_starred(self, ob): ...
    def gen_unpack_spec(self, tpl): ...
    def protect_unpack_sequence(self, target, value): ...
    def gen_unpack_wrapper(self, node, target, ctx: str = "store"): ...
    def gen_lambda(self, args, body): ...
    def gen_del_stmt(self, name_to_del): ...
    def transform_slice(self, slice_): ...
    def isAllowedName(self, node, name) -> None: ...
    def check_function_argument_names(self, node) -> None: ...
    def check_import_names(self, node): ...
    def gen_attr_check(self, node, attr_name): ...
    def visit_Constant(self, node): ...
    def visit_Name(self, node): ...
    def visit_Call(self, node): ...
    def visit_Attribute(self, node): ...
    def visit_Subscript(self, node): ...
    def visit_Assign(self, node): ...
    def visit_AugAssign(self, node): ...
    # Bug in `reportlab`'s source code:
    def visit_While(node): ...  # type: ignore[override]
    def visit_ExceptHandler(self, node): ...
    def visit_With(self, node): ...
    def visit_FunctionDef(self, node): ...
    def visit_Lambda(self, node): ...
    def visit_ClassDef(self, node): ...
    def visit_Import(self, node): ...
    def visit_BinOp(self, node): ...
    visit_ImportFrom = visit_Import  # pyright: ignore[reportAssignmentType]
    visit_For = guard_iter
    visit_comprehension = guard_iter
    def generic_visit(self, node: ast.AST) -> None: ...  # type: ignore[override]
    def not_allowed(self, node: ast.AST) -> NoReturn: ...
    def visit_children(self, node): ...
    def visit(self, node): ...
    visit_Ellipsis = not_allowed
    visit_MatMult = not_allowed
    visit_Exec = not_allowed
    visit_Nonlocal = not_allowed
    visit_AsyncFunctionDef = not_allowed
    visit_Await = not_allowed
    visit_AsyncFor = not_allowed
    visit_AsyncWith = not_allowed
    visit_Print = not_allowed
    visit_Num = visit_children
    visit_Str = visit_children
    visit_Bytes = visit_children
    visit_List = visit_children
    visit_Tuple = visit_children
    visit_Set = visit_children
    visit_Dict = visit_children
    visit_FormattedValue = visit_children
    visit_JoinedStr = visit_children
    visit_NameConstant = visit_children
    visit_Load = visit_children
    visit_Store = visit_children
    visit_Del = visit_children
    visit_Starred = visit_children
    visit_Expression = visit_children
    visit_Expr = visit_children
    visit_UnaryOp = visit_children
    visit_UAdd = visit_children
    visit_USub = visit_children
    visit_Not = visit_children
    visit_Invert = visit_children
    visit_Add = visit_children
    visit_Sub = visit_children
    visit_Mult = visit_children
    visit_Div = visit_children
    visit_FloorDiv = visit_children
    visit_Pow = visit_children
    visit_Mod = visit_children
    visit_LShift = visit_children
    visit_RShift = visit_children
    visit_BitOr = visit_children
    visit_BitXor = visit_children
    visit_BitAnd = visit_children
    visit_BoolOp = visit_children
    visit_And = visit_children
    visit_Or = visit_children
    visit_Compare = visit_children
    visit_Eq = visit_children
    visit_NotEq = visit_children
    visit_Lt = visit_children
    visit_LtE = visit_children
    visit_Gt = visit_children
    visit_GtE = visit_children
    visit_Is = visit_children
    visit_IsNot = visit_children
    visit_In = visit_children
    visit_NotIn = visit_children
    visit_keyword = visit_children
    visit_IfExp = visit_children
    visit_Index = visit_children
    visit_Slice = visit_children
    visit_ExtSlice = visit_children
    visit_ListComp = visit_children
    visit_SetComp = visit_children
    visit_GeneratorExp = visit_children
    visit_DictComp = visit_children
    visit_Raise = visit_children
    visit_Assert = visit_children
    visit_Delete = visit_children
    visit_Pass = visit_children
    visit_alias = visit_children
    visit_If = visit_children
    visit_Break = visit_children
    visit_Continue = visit_children
    visit_Try = visit_children
    visit_TryFinally = visit_children
    visit_TryExcept = visit_children
    visit_withitem = visit_children
    visit_arguments = visit_children
    visit_arg = visit_children
    visit_Return = visit_children
    visit_Yield = visit_children
    visit_YieldFrom = visit_children
    visit_Global = visit_children
    visit_Module = visit_children
    visit_Param = visit_children

def astFormat(node): ...

class __rl_SafeIter__:
    __rl_iter__: Incomplete
    __rl_owner__: Incomplete
    def __init__(self, it, owner) -> None: ...
    def __iter__(self): ...
    def __next__(self): ...
    next = __next__

__rl_safe_builtins__: Incomplete

def safer_globals(g=None): ...

math_log10 = math.log10
__rl_undef__: Incomplete

class __RL_SAFE_ENV__:
    __time_time__ = time.time
    __weakref_ref__: Incomplete
    __slicetype__: Incomplete
    timeout: Incomplete
    allowed_magic_methods: Incomplete
    __rl_gen_range__: Incomplete
    __rl_real_iter__: Incomplete
    real_bi: Incomplete
    bi_replace: Incomplete
    __rl_builtins__: Incomplete
    def __init__(self, timeout=None, allowed_magic_methods=None, allowed_magic_names=None) -> None: ...
    def __rl_type__(self, *args): ...
    def __rl_check__(self) -> None: ...
    def __rl_sd__(self, obj): ...
    def __rl_getiter__(self, it): ...
    def __rl_max__(self, arg, *args, **kwds): ...
    def __rl_min__(self, arg, *args, **kwds): ...
    def __rl_sum__(self, sequence, start: int = 0): ...
    def __rl_enumerate__(self, seq): ...
    def __rl_zip__(self, *args): ...
    def __rl_hasattr__(self, obj, name): ...
    def __rl_filter__(self, f, seq): ...
    def __rl_map__(self, f, seq): ...
    def __rl_any__(self, seq): ...
    def __rl_all__(self, seq): ...
    def __rl_sorted__(self, seq, **kwds): ...
    def __rl_reversed__(self, seq): ...
    def __rl_range__(self, start, *args): ...
    def __rl_set__(self, it): ...
    def __rl_frozenset__(self, it=()): ...
    def __rl_iter_unpack_sequence__(self, it, spec, _getiter_) -> Generator[Incomplete, None, None]: ...
    def __rl_unpack_sequence__(self, it, spec, _getiter_): ...
    def __rl_is_allowed_name__(self, name, crash: bool = True) -> bool: ...
    def __rl_getattr__(self, obj, a, *args): ...
    def __rl_getitem__(self, obj, a): ...
    __rl_tmax__: int
    __rl_max_len__: int
    __rl_max_pow_digits__: int
    def __rl_add__(self, a, b): ...
    def __rl_mult__(self, a, b): ...
    def __rl_pow__(self, a, b): ...
    def __rl_augAssign__(self, op, v, i): ...
    def __rl_apply__(self, func, args, kwds): ...
    def __rl_args_iter__(self, *args): ...
    def __rl_list__(self, it): ...
    def __rl_compile__(
        self, src, fname: str = "<string>", mode: str = "eval", flags: int = 0, inherit: bool = True, visit=None
    ): ...
    __rl_limit__: Incomplete
    def __rl_safe_eval__(
        self, expr, g, l, mode, timeout=None, allowed_magic_methods=None, __frame_depth__: int = 3, allowed_magic_names=None
    ): ...

class __rl_safe_eval__:
    mode: str
    env: Incomplete
    def __init__(self) -> None: ...
    def __call__(self, expr, g=None, l=None, timeout=None, allowed_magic_methods=None, allowed_magic_names=None): ...

class __rl_safe_exec__(__rl_safe_eval__):
    mode: str

def rl_extended_literal_eval(expr, safe_callables=None, safe_names=None): ...

rl_safe_exec: __rl_safe_exec__
rl_safe_eval: __rl_safe_eval__

def __fix_set__(value, default=...): ...
def rl_less_safe_eval(expr, NS): ...



from hvac.api.vault_api_base import VaultApiBase

DEFAULT_MOUNT_POINT: str

class KvV2(VaultApiBase):
    def configure(self, max_versions: int = 10, cas_required=None, delete_version_after: str = "0s", mount_point="secret"): ...
    def read_configuration(self, mount_point="secret"): ...
    def read_secret(self, path, mount_point="secret", raise_on_deleted_version=None): ...
    def read_secret_version(self, path, version=None, mount_point="secret", raise_on_deleted_version=None): ...
    def create_or_update_secret(self, path, secret, cas=None, mount_point="secret"): ...
    def patch(self, path, secret, mount_point="secret"): ...
    def delete_latest_version_of_secret(self, path, mount_point="secret"): ...
    def delete_secret_versions(self, path, versions, mount_point="secret"): ...
    def undelete_secret_versions(self, path, versions, mount_point="secret"): ...
    def destroy_secret_versions(self, path, versions, mount_point="secret"): ...
    def list_secrets(self, path, mount_point="secret"): ...
    def read_secret_metadata(self, path, mount_point="secret"): ...
    def update_metadata(
        self,
        path,
        max_versions=None,
        cas_required=None,
        delete_version_after: str = "0s",
        mount_point="secret",
        custom_metadata=None,
    ): ...
    def delete_metadata_and_all_versions(self, path, mount_point="secret"): ...



from typing import Any, Generic, NoReturn, TypeVar
from typing_extensions import Self

from docker import APIClient

_T = TypeVar("_T", bound=Model)

class Model:
    id_attribute: str
    client: APIClient | None
    collection: Collection[Self] | None
    attrs: dict[str, Any]
    def __init__(
        self, attrs: dict[str, Any] | None = None, client: APIClient | None = None, collection: Collection[Self] | None = None
    ) -> None: ...
    def __eq__(self, other) -> bool: ...
    def __hash__(self) -> int: ...
    @property
    def id(self) -> str | None: ...
    @property
    def short_id(self) -> str: ...
    def reload(self) -> None: ...

class Collection(Generic[_T]):
    model: type[_T]
    client: APIClient
    def __init__(self, client: APIClient | None = None) -> None: ...
    def __call__(self, *args, **kwargs) -> NoReturn: ...
    def list(self) -> list[_T]: ...
    def get(self, key: str) -> _T: ...
    def create(self, attrs: Any | None = None) -> _T: ...
    def prepare_model(self, attrs: Model | dict[str, Any]) -> _T: ...



from subprocess import CalledProcessError

class StoreError(RuntimeError): ...
class CredentialsNotFound(StoreError): ...
class InitializationError(StoreError): ...

def process_store_error(cpe: CalledProcessError, program: str) -> Exception: ...



import sys

if sys.platform == "win32":
    from collections.abc import Sequence
    from socket import AddressFamily, SocketKind
    from typing import Final

    version: Final[int]
    ABOVE_NORMAL_PRIORITY_CLASS: Final = 32768
    BELOW_NORMAL_PRIORITY_CLASS: Final = 16384
    HIGH_PRIORITY_CLASS: Final = 128
    IDLE_PRIORITY_CLASS: Final = 64
    NORMAL_PRIORITY_CLASS: Final = 32
    REALTIME_PRIORITY_CLASS: Final = 256
    MIB_TCP_STATE_CLOSED: Final = 1
    MIB_TCP_STATE_CLOSING: Final = 9
    MIB_TCP_STATE_CLOSE_WAIT: Final = 8
    MIB_TCP_STATE_LISTEN: Final = 2
    MIB_TCP_STATE_ESTAB: Final = 5
    MIB_TCP_STATE_SYN_SENT: Final = 3
    MIB_TCP_STATE_SYN_RCVD: Final = 4
    MIB_TCP_STATE_FIN_WAIT1: Final = 6
    MIB_TCP_STATE_FIN_WAIT2: Final = 7
    MIB_TCP_STATE_LAST_ACK: Final = 10
    MIB_TCP_STATE_TIME_WAIT: Final = 11
    MIB_TCP_STATE_DELETE_TCB: Final = 12
    PSUTIL_CONN_NONE: Final = 128
    INFINITE: Final[int]
    ERROR_ACCESS_DENIED: Final = 5
    ERROR_INVALID_NAME: Final = 123
    ERROR_SERVICE_DOES_NOT_EXIST: Final = 1060
    ERROR_PRIVILEGE_NOT_HELD: Final = 1314
    WINVER: Final[int]
    WINDOWS_VISTA: Final = 60
    WINDOWS_7: Final = 61
    WINDOWS_8: Final = 62
    WINDOWS_8_1: Final = 63
    WINDOWS_10: Final = 100

    class TimeoutAbandoned(Exception): ...
    class TimeoutExpired(Exception): ...

    def proc_cmdline(pid: int, use_peb: bool = True) -> list[str]: ...
    def proc_cpu_affinity_get(pid: int, /) -> int: ...
    def proc_cpu_affinity_set(pid: int, mask: int, /) -> None: ...
    def proc_cwd(pid: int, /) -> str: ...
    def proc_environ(pid: int, /) -> str: ...
    def proc_exe(pid: int, /) -> str: ...
    def proc_io_counters(pid: int, /) -> tuple[int, int, int, int, int, int]: ...
    def proc_io_priority_get(pid: int, /) -> int: ...
    def proc_io_priority_set(pid: int, priority: int, /) -> None: ...
    def proc_is_suspended(pid: int, /) -> bool: ...
    def proc_kill(pid: int, /) -> None: ...
    def proc_memory_info(pid: int, /) -> tuple[int, int, int, int, int, int, int, int, int, int]: ...
    def proc_memory_maps(pid: int, /) -> list[tuple[int, str, str, int]]: ...
    def proc_memory_uss(pid: int, /) -> int: ...
    def proc_num_handles(pid: int, /) -> int: ...
    def proc_open_files(pid: int, /) -> list[str]: ...
    def proc_priority_get(pid: int, /) -> int: ...
    def proc_priority_set(pid: int, priority: int, /) -> None: ...
    def proc_suspend_or_resume(pid: int, suspend: bool | None, /) -> None: ...
    def proc_threads(pid: int, /) -> list[tuple[int, float, float]]: ...
    def proc_times(pid: int, /) -> tuple[float, float, float]: ...
    def proc_username(pid: int, /) -> tuple[str, str]: ...
    def proc_wait(pid: int, timeout: int, /) -> int | None: ...
    def proc_info(
        pid: int, /
    ) -> tuple[int, int, int, int, int, int, int, int, int, int, int, int, int, int, int, int, int, int, int, int, int, int]: ...
    def boot_time() -> float: ...
    def cpu_count_cores() -> int | None: ...
    def cpu_count_logical() -> int | None: ...
    def cpu_freq() -> tuple[int, int]: ...
    def cpu_stats() -> tuple[int, int, int, int]: ...
    def cpu_times() -> tuple[float, float, float]: ...
    def disk_io_counters() -> dict[str, tuple[int, int, int, int, int, int]]: ...
    def disk_partitions(all: bool, /) -> list[tuple[str, str, str, str]]: ...
    def disk_usage(path: str, /) -> tuple[int, int, int]: ...
    def getloadavg() -> tuple[float, float, float]: ...
    def getpagesize() -> int: ...
    def swap_percent() -> float: ...
    def init_loadavg_counter() -> None: ...
    def net_connections(
        pid: int, af_filter: Sequence[AddressFamily | int | None], type_filter: Sequence[SocketKind | int | None], /
    ) -> list[tuple[int, int, int, tuple[str | None, int], tuple[str | None, int], int, int]]: ...
    def net_if_addrs() -> list[tuple[str, int, str, str | None, None, None]]: ...
    def net_if_stats() -> dict[str, tuple[bool, int, int, int]]: ...
    def net_io_counters() -> dict[str, tuple[int, int, int, int, int, int, int, int]]: ...
    def per_cpu_times() -> list[tuple[float, float, float, float, float]]: ...
    def pid_exists(pid: int, /) -> bool: ...
    def pids() -> list[int]: ...
    def ppid_map() -> dict[int, int]: ...
    def sensors_battery() -> tuple[int, int, int, int]: ...
    def users() -> list[tuple[str, str | None, float]]: ...
    def virtual_mem() -> tuple[int, int, int, int]: ...
    def winservice_enumerate() -> list[tuple[str, str]]: ...
    def winservice_query_config(service_name: str, /) -> tuple[str, str, str, str]: ...
    def winservice_query_descr(service_name: str, /) -> str: ...
    def winservice_query_status(service_name: str, /) -> tuple[str, int] | str: ...
    def winservice_start(service_name: str, /) -> None: ...
    def winservice_stop(service_name: str, /) -> None: ...
    def QueryDosDevice(device_path: str, /) -> str: ...
    def check_pid_range(pid: int, /) -> None: ...
    def set_debug(value: bool, /) -> None: ...



import datetime
from typing import Final

from ..core import WesternCalendar

QUEENS_BIRTHDAY_EXCEPTIONS: Final[dict[int, datetime.date]]

class CaymanIslands(WesternCalendar):
    def get_national_heroes_day(self, year: int) -> datetime.date: ...
    def get_discovery_day(self, year: int) -> datetime.date: ...
    def get_queens_birthday(self, year: int) -> datetime.date: ...
    def get_constitution_day(self, year: int) -> datetime.date: ...
    def get_remembrance_day(self, year: int) -> datetime.date: ...



from typing import Final
from typing_extensions import deprecated

__all__ = ["available", "enable", "disable", "enabled"]

available: Final = True
enabled: Final = True

@deprecated("Function `enable` is deprecated and no longer has any effect. Speedups are always available.")
def enable() -> None: ...
@deprecated("Function `disable` is deprecated and no longer has any effect. Speedups are always available.")
def disable() -> None: ...



from _typeshed import Incomplete

from ..rest import RestClientOptions
from ..types import TimeoutType

class SelfServiceProfiles:
    def __init__(
        self,
        domain: str,
        token: str,
        telemetry: bool = True,
        timeout: TimeoutType = 5.0,
        protocol: str = "https",
        rest_options: RestClientOptions | None = None,
    ) -> None: ...
    def all(self, page: int = 0, per_page: int = 25, include_totals: bool = True) -> list[dict[str, Incomplete]]: ...
    def create(self, body: dict[str, Incomplete]) -> dict[str, Incomplete]: ...
    def get(self, profile_id: str) -> dict[str, Incomplete]: ...
    def delete(self, profile_id: str) -> None: ...
    def update(self, profile_id: str, body: dict[str, Incomplete]) -> dict[str, Incomplete]: ...
    def get_custom_text(self, profile_id: str, language: str, page: str) -> dict[str, Incomplete]: ...
    def update_custom_text(
        self, profile_id: str, language: str, page: str, body: dict[str, Incomplete]
    ) -> dict[str, Incomplete]: ...
    def create_sso_ticket(self, profile_id: str, body: dict[str, Incomplete]) -> dict[str, Incomplete]: ...
    def revoke_sso_ticket(self, profile_id: str, ticket_id: str) -> None: ...



class InvalidCalendar(ValueError): ...
class IncompleteComponent(ValueError): ...
class IncompleteAlarmInformation(ValueError): ...
class LocalTimezoneMissing(IncompleteAlarmInformation): ...
class ComponentEndMissing(IncompleteAlarmInformation): ...
class ComponentStartMissing(IncompleteAlarmInformation): ...
class FeatureWillBeRemovedInFutureVersion(DeprecationWarning): ...
class WillBeRemovedInVersion7(FeatureWillBeRemovedInFutureVersion): ...

__all__ = [
    "InvalidCalendar",
    "IncompleteComponent",
    "IncompleteAlarmInformation",
    "LocalTimezoneMissing",
    "ComponentEndMissing",
    "ComponentStartMissing",
    "FeatureWillBeRemovedInFutureVersion",
    "WillBeRemovedInVersion7",
]



import sys
from _typeshed import (
    BytesPath,
    OpenBinaryMode,
    OpenBinaryModeReading,
    OpenBinaryModeUpdating,
    OpenBinaryModeWriting,
    OpenTextMode,
    StrOrBytesPath,
    StrPath,
)
from asyncio import AbstractEventLoop
from concurrent.futures import Executor
from typing import AnyStr, Literal, overload

from ..base import AiofilesContextManager
from ..threadpool.binary import AsyncBufferedIOBase, AsyncBufferedReader, AsyncFileIO
from ..threadpool.text import AsyncTextIOWrapper

# Text mode: always returns AsyncTextIOWrapper
@overload
def TemporaryFile(
    mode: OpenTextMode,
    buffering: int = -1,
    encoding: str | None = None,
    newline: str | None = None,
    suffix: AnyStr | None = None,
    prefix: AnyStr | None = None,
    dir: StrOrBytesPath | None = None,
    loop: AbstractEventLoop | None = None,
    executor: Executor | None = None,
) -> AiofilesContextManager[AsyncTextIOWrapper]: ...

# Unbuffered binary: returns a FileIO
@overload
def TemporaryFile(
    mode: OpenBinaryMode,
    buffering: Literal[0],
    encoding: None = None,
    newline: None = None,
    suffix: AnyStr | None = None,
    prefix: AnyStr | None = None,
    dir: StrOrBytesPath | None = None,
    loop: AbstractEventLoop | None = None,
    executor: Executor | None = None,
) -> AiofilesContextManager[AsyncFileIO]: ...

# Buffered binary reading/updating: AsyncBufferedReader
@overload
def TemporaryFile(
    mode: OpenBinaryModeReading | OpenBinaryModeUpdating = "w+b",
    buffering: Literal[-1, 1] = -1,
    encoding: None = None,
    newline: None = None,
    suffix: AnyStr | None = None,
    prefix: AnyStr | None = None,
    dir: StrOrBytesPath | None = None,
    loop: AbstractEventLoop | None = None,
    executor: Executor | None = None,
) -> AiofilesContextManager[AsyncBufferedReader]: ...

# Buffered binary writing: AsyncBufferedIOBase
@overload
def TemporaryFile(
    mode: OpenBinaryModeWriting,
    buffering: Literal[-1, 1] = -1,
    encoding: None = None,
    newline: None = None,
    suffix: AnyStr | None = None,
    prefix: AnyStr | None = None,
    dir: StrOrBytesPath | None = None,
    loop: AbstractEventLoop | None = None,
    executor: Executor | None = None,
) -> AiofilesContextManager[AsyncBufferedIOBase]: ...

# 3.12 added `delete_on_close`
if sys.version_info >= (3, 12):
    # Text mode: always returns AsyncTextIOWrapper
    @overload
    def NamedTemporaryFile(
        mode: OpenTextMode,
        buffering: int = -1,
        encoding: str | None = None,
        newline: str | None = None,
        suffix: AnyStr | None = None,
        prefix: AnyStr | None = None,
        dir: StrOrBytesPath | None = None,
        delete: bool = True,
        delete_on_close: bool = True,
        loop: AbstractEventLoop | None = None,
        executor: Executor | None = None,
    ) -> AiofilesContextManager[AsyncTextIOWrapper]: ...

    # Unbuffered binary: returns a FileIO
    @overload
    def NamedTemporaryFile(
        mode: OpenBinaryMode,
        buffering: Literal[0],
        encoding: None = None,
        newline: None = None,
        suffix: AnyStr | None = None,
        prefix: AnyStr | None = None,
        dir: StrOrBytesPath | None = None,
        delete: bool = True,
        delete_on_close: bool = True,
        loop: AbstractEventLoop | None = None,
        executor: Executor | None = None,
    ) -> AiofilesContextManager[AsyncFileIO]: ...

    # Buffered binary reading/updating: AsyncBufferedReader
    @overload
    def NamedTemporaryFile(
        mode: OpenBinaryModeReading | OpenBinaryModeUpdating = "w+b",
        buffering: Literal[-1, 1] = -1,
        encoding: None = None,
        newline: None = None,
        suffix: AnyStr | None = None,
        prefix: AnyStr | None = None,
        dir: StrOrBytesPath | None = None,
        delete: bool = True,
        delete_on_close: bool = True,
        loop: AbstractEventLoop | None = None,
        executor: Executor | None = None,
    ) -> AiofilesContextManager[AsyncBufferedReader]: ...

    # Buffered binary writing: AsyncBufferedIOBase
    @overload
    def NamedTemporaryFile(
        mode: OpenBinaryModeWriting,
        buffering: Literal[-1, 1] = -1,
        encoding: None = None,
        newline: None = None,
        suffix: AnyStr | None = None,
        prefix: AnyStr | None = None,
        dir: StrOrBytesPath | None = None,
        delete: bool = True,
        delete_on_close: bool = True,
        loop: AbstractEventLoop | None = None,
        executor: Executor | None = None,
    ) -> AiofilesContextManager[AsyncBufferedIOBase]: ...

else:
    # Text mode: always returns AsyncTextIOWrapper
    @overload
    def NamedTemporaryFile(
        mode: OpenTextMode,
        buffering: int = -1,
        encoding: str | None = None,
        newline: str | None = None,
        suffix: AnyStr | None = None,
        prefix: AnyStr | None = None,
        dir: StrOrBytesPath | None = None,
        delete: bool = True,
        loop: AbstractEventLoop | None = None,
        executor: Executor | None = None,
    ) -> AiofilesContextManager[AsyncTextIOWrapper]: ...

    # Unbuffered binary: returns a FileIO
    @overload
    def NamedTemporaryFile(
        mode: OpenBinaryMode,
        buffering: Literal[0],
        encoding: None = None,
        newline: None = None,
        suffix: AnyStr | None = None,
        prefix: AnyStr | None = None,
        dir: StrOrBytesPath | None = None,
        delete: bool = True,
        loop: AbstractEventLoop | None = None,
        executor: Executor | None = None,
    ) -> AiofilesContextManager[AsyncFileIO]: ...

    # Buffered binary reading/updating: AsyncBufferedReader
    @overload
    def NamedTemporaryFile(
        mode: OpenBinaryModeReading | OpenBinaryModeUpdating = "w+b",
        buffering: Literal[-1, 1] = -1,
        encoding: None = None,
        newline: None = None,
        suffix: AnyStr | None = None,
        prefix: AnyStr | None = None,
        dir: StrOrBytesPath | None = None,
        delete: bool = True,
        loop: AbstractEventLoop | None = None,
        executor: Executor | None = None,
    ) -> AiofilesContextManager[AsyncBufferedReader]: ...

    # Buffered binary writing: AsyncBufferedIOBase
    @overload
    def NamedTemporaryFile(
        mode: OpenBinaryModeWriting,
        buffering: Literal[-1, 1] = -1,
        encoding: None = None,
        newline: None = None,
        suffix: AnyStr | None = None,
        prefix: AnyStr | None = None,
        dir: StrOrBytesPath | None = None,
        delete: bool = True,
        loop: AbstractEventLoop | None = None,
        executor: Executor | None = None,
    ) -> AiofilesContextManager[AsyncBufferedIOBase]: ...

# Text mode: always returns AsyncTextIOWrapper
@overload
def SpooledTemporaryFile(
    max_size: int = 0,
    *,
    mode: OpenTextMode,
    buffering: int = -1,
    encoding: str | None = None,
    newline: str | None = None,
    suffix: AnyStr | None = None,
    prefix: AnyStr | None = None,
    dir: StrOrBytesPath | None = None,
    loop: AbstractEventLoop | None = None,
    executor: Executor | None = None,
) -> AiofilesContextManager[AsyncTextIOWrapper]: ...
@overload
def SpooledTemporaryFile(
    max_size: int,
    mode: OpenTextMode,
    buffering: int = -1,
    encoding: str | None = None,
    newline: str | None = None,
    suffix: AnyStr | None = None,
    prefix: AnyStr | None = None,
    dir: StrOrBytesPath | None = None,
    loop: AbstractEventLoop | None = None,
    executor: Executor | None = None,
) -> AiofilesContextManager[AsyncTextIOWrapper]: ...

# Unbuffered binary: returns a FileIO
@overload
def SpooledTemporaryFile(
    max_size: int = 0,
    mode: OpenBinaryMode = "w+b",
    *,
    buffering: Literal[0],
    encoding: None = None,
    newline: None = None,
    suffix: AnyStr | None = None,
    prefix: AnyStr | None = None,
    dir: StrOrBytesPath | None = None,
    loop: AbstractEventLoop | None = None,
    executor: Executor | None = None,
) -> AiofilesContextManager[AsyncFileIO]: ...
@overload
def SpooledTemporaryFile(
    max_size: int,
    mode: OpenBinaryMode,
    buffering: Literal[0],
    encoding: None = None,
    newline: None = None,
    suffix: AnyStr | None = None,
    prefix: AnyStr | None = None,
    dir: StrOrBytesPath | None = None,
    loop: AbstractEventLoop | None = None,
    executor: Executor | None = None,
) -> AiofilesContextManager[AsyncFileIO]: ...

# Buffered binary reading/updating: AsyncBufferedReader
@overload
def SpooledTemporaryFile(
    max_size: int = 0,
    mode: OpenBinaryModeReading | OpenBinaryModeUpdating = "w+b",
    buffering: Literal[-1, 1] = -1,
    encoding: None = None,
    newline: None = None,
    suffix: AnyStr | None = None,
    prefix: AnyStr | None = None,
    dir: StrOrBytesPath | None = None,
    loop: AbstractEventLoop | None = None,
    executor: Executor | None = None,
) -> AiofilesContextManager[AsyncBufferedReader]: ...

# Buffered binary writing: AsyncBufferedIOBase
@overload
def SpooledTemporaryFile(
    max_size: int = 0,
    *,
    mode: OpenBinaryModeWriting,
    buffering: Literal[-1, 1] = -1,
    encoding: None = None,
    newline: None = None,
    suffix: AnyStr | None = None,
    prefix: AnyStr | None = None,
    dir: StrOrBytesPath | None = None,
    loop: AbstractEventLoop | None = None,
    executor: Executor | None = None,
) -> AiofilesContextManager[AsyncBufferedIOBase]: ...
@overload
def SpooledTemporaryFile(
    max_size: int,
    mode: OpenBinaryModeWriting,
    buffering: Literal[-1, 1] = -1,
    encoding: None = None,
    newline: None = None,
    suffix: AnyStr | None = None,
    prefix: AnyStr | None = None,
    dir: StrOrBytesPath | None = None,
    loop: AbstractEventLoop | None = None,
    executor: Executor | None = None,
) -> AiofilesContextManager[AsyncBufferedIOBase]: ...
@overload
def TemporaryDirectory(
    suffix: str | None = None,
    prefix: str | None = None,
    dir: StrPath | None = None,
    loop: AbstractEventLoop | None = None,
    executor: Executor | None = None,
) -> AiofilesContextManagerTempDir: ...
@overload
def TemporaryDirectory(
    suffix: bytes | None = None,
    prefix: bytes | None = None,
    dir: BytesPath | None = None,
    loop: AbstractEventLoop | None = None,
    executor: Executor | None = None,
) -> AiofilesContextManagerTempDir: ...

class AiofilesContextManagerTempDir(AiofilesContextManager[str]):
    async def __aenter__(self) -> str: ...

__all__ = ["NamedTemporaryFile", "TemporaryFile", "SpooledTemporaryFile", "TemporaryDirectory"]



from _typeshed import Unused
from typing import TypedDict, type_check_only

from .sources import Source

@type_check_only
class _Line(TypedDict):
    line: int

@type_check_only
class _BreakpointLocationsResult(TypedDict):
    breakpoints: list[_Line]

def breakpoint_locations(
    *, source: Source, line: int, endLine: int | None = None, **extra: Unused
) -> _BreakpointLocationsResult: ...



from collections.abc import Iterable

from networkx.classes.graph import Graph, _Node
from networkx.utils.backends import _dispatchable

__all__ = [
    "boundary_expansion",
    "conductance",
    "cut_size",
    "edge_expansion",
    "mixing_expansion",
    "node_expansion",
    "normalized_cut_size",
    "volume",
]

@_dispatchable
def cut_size(G: Graph[_Node], S: Iterable[_Node], T: Iterable[_Node] | None = None, weight: str | None = None): ...
@_dispatchable
def volume(G: Graph[_Node], S: Iterable[_Node], weight: str | None = None): ...
@_dispatchable
def normalized_cut_size(G: Graph[_Node], S: Iterable[_Node], T: Iterable[_Node] | None = None, weight: str | None = None): ...
@_dispatchable
def conductance(G: Graph[_Node], S: Iterable[_Node], T: Iterable[_Node] | None = None, weight: str | None = None): ...
@_dispatchable
def edge_expansion(G: Graph[_Node], S: Iterable[_Node], T: Iterable[_Node] | None = None, weight: str | None = None): ...
@_dispatchable
def mixing_expansion(G: Graph[_Node], S: Iterable[_Node], T: Iterable[_Node] | None = None, weight: str | None = None): ...
@_dispatchable
def node_expansion(G: Graph[_Node], S: Iterable[_Node]): ...
@_dispatchable
def boundary_expansion(G: Graph[_Node], S: Iterable[_Node]): ...



from _typeshed import Incomplete, StrOrBytesPath, StrPath, SupportsGetItem, Unused
from collections.abc import Callable, Iterable, Mapping
from optparse import OptionParser
from typing import Final, Literal

this_dir: str

class FilterParameters:
    Name: Incomplete
    Description: Incomplete
    Path: Incomplete
    Server: Incomplete
    AddExtensionFile: bool
    AddExtensionFile_Enabled: bool
    AddExtensionFile_GroupID: Incomplete
    AddExtensionFile_CanDelete: bool
    AddExtensionFile_Description: Incomplete
    def __init__(self, **kw) -> None: ...

class VirtualDirParameters:
    Name: Incomplete
    Description: Incomplete
    AppProtection: Incomplete
    Headers: Incomplete
    Path: Incomplete
    Type: Incomplete
    AccessExecute: Incomplete
    AccessRead: Incomplete
    AccessWrite: Incomplete
    AccessScript: Incomplete
    ContentIndexed: Incomplete
    EnableDirBrowsing: Incomplete
    EnableDefaultDoc: Incomplete
    DefaultDoc: Incomplete
    ScriptMaps: list[ScriptMapParams]
    ScriptMapUpdate: str
    Server: Incomplete
    def __init__(self, **kw) -> None: ...
    def is_root(self) -> bool: ...
    def split_path(self) -> list[str]: ...

class ScriptMapParams:
    Extension: Incomplete
    Module: Incomplete
    Flags: int
    Verbs: str
    AddExtensionFile: bool
    AddExtensionFile_Enabled: bool
    AddExtensionFile_GroupID: Incomplete
    AddExtensionFile_CanDelete: bool
    AddExtensionFile_Description: Incomplete
    def __init__(self, **kw) -> None: ...

class ISAPIParameters:
    ServerName: Incomplete
    Filters: list[FilterParameters]
    VirtualDirs: list[VirtualDirParameters]
    def __init__(self, **kw) -> None: ...

verbose: int

def log(level: int, what: object) -> None: ...

class InstallationError(Exception): ...
class ItemNotFound(InstallationError): ...
class ConfigurationError(InstallationError): ...

def FindPath(options, server: str | bytes | bytearray, name: str) -> str: ...
def LocateWebServerPath(description: str): ...
def GetWebServer(description: str | None = None): ...
def LoadWebServer(path): ...
def FindWebServer(options, server_desc: str | bytes | bytearray | None) -> str: ...
def split_path(path: str) -> list[str]: ...
def CreateDirectory(params, options): ...
def AssignScriptMaps(script_maps: Iterable[ScriptMapParams], target, update: str = "replace") -> None: ...
def get_unique_items(sequence, reference): ...
def CreateISAPIFilter(filterParams, options): ...
def DeleteISAPIFilter(filterParams, options) -> None: ...
def AddExtensionFiles(params, options) -> None: ...
def DeleteExtensionFileRecords(params, options) -> None: ...
def CheckLoaderModule(dll_name: StrOrBytesPath) -> None: ...
def Install(params, options) -> None: ...
def RemoveDirectory(params, options) -> None: ...
def RemoveScriptMaps(vd_params, options) -> None: ...
def Uninstall(params, options) -> None: ...
def GetLoaderModuleName(mod_name: StrPath, check_module: bool | None = None) -> str: ...
def InstallModule(conf_module_name: StrPath, params, options, log: Callable[[int, str], Unused] = ...) -> None: ...
def UninstallModule(conf_module_name: StrPath, params, options, log: Callable[[int, str], Unused] = ...) -> None: ...

standard_arguments: Final[dict[Literal["install", "remove"], Callable[..., Incomplete]]]

def build_usage(handler_map: Mapping[str, object]) -> str: ...
def MergeStandardOptions(options, params) -> None: ...
def HandleCommandLine(
    params,
    argv: SupportsGetItem[int, str] | None = None,
    conf_module_name: str | None = None,
    default_arg: str = "install",
    opt_parser: OptionParser | None = None,
    custom_arg_handlers: Mapping[str, object] = {},
) -> None: ...



from typing import ClassVar

from sassutils.builder import Manifest as Manifest
from setuptools import Command, Distribution

def validate_manifests(dist: Distribution, attr: str, value: object) -> None: ...

class build_sass(Command):
    description: str
    user_options: ClassVar[list[tuple[str, str, str]]]
    package_dir: dict[str, str] | None
    output_style: str
    def initialize_options(self) -> None: ...
    def finalize_options(self) -> None: ...
    def run(self) -> None: ...
    def get_package_dir(self, package: str) -> str: ...

__all__ = ("build_sass", "validate_manifests")



import logging

from hvac.api.vault_api_base import VaultApiBase

logger: logging.Logger

class Identity(VaultApiBase):
    def create_or_update_entity(
        self, name, entity_id=None, metadata=None, policies=None, disabled=None, mount_point: str = "identity"
    ): ...
    def create_or_update_entity_by_name(
        self, name, metadata=None, policies=None, disabled=None, mount_point: str = "identity"
    ): ...
    def read_entity(self, entity_id, mount_point: str = "identity"): ...
    def read_entity_by_name(self, name, mount_point: str = "identity"): ...
    def update_entity(self, entity_id, name=None, metadata=None, policies=None, disabled=None, mount_point: str = "identity"): ...
    def delete_entity(self, entity_id, mount_point: str = "identity"): ...
    def delete_entity_by_name(self, name, mount_point: str = "identity"): ...
    def list_entities(self, method: str = "LIST", mount_point: str = "identity"): ...
    def list_entities_by_name(self, method: str = "LIST", mount_point: str = "identity"): ...
    def merge_entities(
        self, from_entity_ids, to_entity_id, force=None, mount_point: str = "identity", conflicting_alias_ids_to_keep=None
    ): ...
    def create_or_update_entity_alias(self, name, canonical_id, mount_accessor, alias_id=None, mount_point: str = "identity"): ...
    def read_entity_alias(self, alias_id, mount_point: str = "identity"): ...
    def update_entity_alias(self, alias_id, name, canonical_id, mount_accessor, mount_point: str = "identity"): ...
    def list_entity_aliases(self, method: str = "LIST", mount_point: str = "identity"): ...
    def delete_entity_alias(self, alias_id, mount_point: str = "identity"): ...
    @staticmethod
    def validate_member_id_params_for_group_type(group_type, params, member_group_ids, member_entity_ids): ...
    def create_or_update_group(
        self,
        name,
        group_id=None,
        group_type: str = "internal",
        metadata=None,
        policies=None,
        member_group_ids=None,
        member_entity_ids=None,
        mount_point: str = "identity",
    ): ...
    def read_group(self, group_id, mount_point: str = "identity"): ...
    def update_group(
        self,
        group_id,
        name,
        group_type: str = "internal",
        metadata=None,
        policies=None,
        member_group_ids=None,
        member_entity_ids=None,
        mount_point: str = "identity",
    ): ...
    def delete_group(self, group_id, mount_point: str = "identity"): ...
    def list_groups(self, method: str = "LIST", mount_point: str = "identity"): ...
    def list_groups_by_name(self, method: str = "LIST", mount_point: str = "identity"): ...
    def create_or_update_group_by_name(
        self,
        name,
        group_type: str = "internal",
        metadata=None,
        policies=None,
        member_group_ids=None,
        member_entity_ids=None,
        mount_point: str = "identity",
    ): ...
    def read_group_by_name(self, name, mount_point: str = "identity"): ...
    def delete_group_by_name(self, name, mount_point: str = "identity"): ...
    def create_or_update_group_alias(
        self, name, alias_id=None, mount_accessor=None, canonical_id=None, mount_point: str = "identity"
    ): ...
    def update_group_alias(self, entity_id, name, mount_accessor=None, canonical_id=None, mount_point="identity"): ...
    def read_group_alias(self, alias_id, mount_point: str = "identity"): ...
    def delete_group_alias(self, entity_id, mount_point: str = "identity"): ...
    def list_group_aliases(self, method: str = "LIST", mount_point: str = "identity"): ...
    def lookup_entity(
        self, name=None, entity_id=None, alias_id=None, alias_name=None, alias_mount_accessor=None, mount_point: str = "identity"
    ): ...
    def lookup_group(
        self, name=None, group_id=None, alias_id=None, alias_name=None, alias_mount_accessor=None, mount_point: str = "identity"
    ): ...
    def configure_tokens_backend(self, issuer=None, mount_point: str = "identity"): ...
    def read_tokens_backend_configuration(self, mount_point: str = "identity"): ...
    def create_named_key(
        self,
        name,
        rotation_period: str = "24h",
        verification_ttl: str = "24h",
        allowed_client_ids=None,
        algorithm: str = "RS256",
        mount_point: str = "identity",
    ): ...
    def read_named_key(self, name, mount_point: str = "identity"): ...
    def delete_named_key(self, name, mount_point: str = "identity"): ...
    def list_named_keys(self, mount_point: str = "identity"): ...
    def rotate_named_key(self, name, verification_ttl, mount_point: str = "identity"): ...
    def create_or_update_role(
        self, name, key, template=None, client_id=None, ttl: str = "24h", mount_point: str = "identity"
    ): ...
    def read_role(self, name, mount_point: str = "identity"): ...
    def delete_role(self, name, mount_point: str = "identity"): ...
    def list_roles(self, mount_point: str = "identity"): ...
    def generate_signed_id_token(self, name, mount_point: str = "identity"): ...
    def introspect_signed_id_token(self, token, client_id=None, mount_point: str = "identity"): ...
    def read_well_known_configurations(self, mount_point: str = "identity"): ...
    def read_active_public_keys(self, mount_point: str = "identity"): ...



from _typeshed import Incomplete
from collections.abc import Iterable

from networkx.classes.graph import Graph, _Node
from networkx.utils.backends import _dispatchable

__all__ = ["harmonic_centrality"]

@_dispatchable
def harmonic_centrality(
    G: Graph[_Node], nbunch: Iterable[Incomplete] | None = None, distance=None, sources: Iterable[Incomplete] | None = None
) -> dict[Incomplete, int]: ...



from typing import TypedDict, type_check_only

@type_check_only
class _Versions(TypedDict):
    dirty: bool
    error: None
    full_revisionid: str
    version: str

version_json: str

def get_versions() -> _Versions: ...



from _typeshed import ReadableBuffer
from asyncio import transports
from typing import Any

# Keep asyncio.__all__ updated with any changes to __all__ here
__all__ = ("BaseProtocol", "Protocol", "DatagramProtocol", "SubprocessProtocol", "BufferedProtocol")

class BaseProtocol:
    __slots__ = ()
    def connection_made(self, transport: transports.BaseTransport) -> None: ...
    def connection_lost(self, exc: Exception | None) -> None: ...
    def pause_writing(self) -> None: ...
    def resume_writing(self) -> None: ...

class Protocol(BaseProtocol):
    # Need annotation or mypy will complain about 'Cannot determine type of "__slots__" in base class'
    __slots__: tuple[str, ...] = ()
    def data_received(self, data: bytes) -> None: ...
    def eof_received(self) -> bool | None: ...

class BufferedProtocol(BaseProtocol):
    __slots__ = ()
    def get_buffer(self, sizehint: int) -> ReadableBuffer: ...
    def buffer_updated(self, nbytes: int) -> None: ...
    def eof_received(self) -> bool | None: ...

class DatagramProtocol(BaseProtocol):
    __slots__ = ()
    def connection_made(self, transport: transports.DatagramTransport) -> None: ...  # type: ignore[override]
    # addr can be a tuple[int, int] for some unusual protocols like socket.AF_NETLINK.
    # Use tuple[str | Any, int] to not cause typechecking issues on most usual cases.
    # This could be improved by using tuple[AnyOf[str, int], int] if the AnyOf feature is accepted.
    # See https://github.com/python/typing/issues/566
    def datagram_received(self, data: bytes, addr: tuple[str | Any, int]) -> None: ...
    def error_received(self, exc: Exception) -> None: ...

class SubprocessProtocol(BaseProtocol):
    __slots__: tuple[str, ...] = ()
    def pipe_data_received(self, fd: int, data: bytes) -> None: ...
    def pipe_connection_lost(self, fd: int, exc: Exception | None) -> None: ...
    def process_exited(self) -> None: ...



from _typeshed import ConvertibleToFloat, SliceableBuffer, Unused
from collections.abc import Iterable, Sequence
from typing import Final, TypeVar

from Xlib.display import Display
from Xlib.protocol import display, request, rq
from Xlib.xobject import drawable, resource

_T = TypeVar("_T")

extname: Final = "XInputExtension"
PropertyDeleted: Final = 0
PropertyCreated: Final = 1
PropertyModified: Final = 2
NotifyNormal: Final = 0
NotifyGrab: Final = 1
NotifyUngrab: Final = 2
NotifyWhileGrabbed: Final = 3
NotifyPassiveGrab: Final = 4
NotifyPassiveUngrab: Final = 5
NotifyAncestor: Final = 0
NotifyVirtual: Final = 1
NotifyInferior: Final = 2
NotifyNonlinear: Final = 3
NotifyNonlinearVirtual: Final = 4
NotifyPointer: Final = 5
NotifyPointerRoot: Final = 6
NotifyDetailNone: Final = 7
GrabtypeButton: Final = 0
GrabtypeKeycode: Final = 1
GrabtypeEnter: Final = 2
GrabtypeFocusIn: Final = 3
GrabtypeTouchBegin: Final = 4
AnyModifier: Final = 0x80000000
AnyButton: Final = 0
AnyKeycode: Final = 0
AsyncDevice: Final = 0
SyncDevice: Final = 1
ReplayDevice: Final = 2
AsyncPairedDevice: Final = 3
AsyncPair: Final = 4
SyncPair: Final = 5
SlaveSwitch: Final = 1
DeviceChange: Final = 2
MasterAdded: Final = 0x01
MasterRemoved: Final = 0x02
SlaveAdded: Final = 0x04
SlaveRemoved: Final = 0x08
SlaveAttached: Final = 0x10
SlaveDetached: Final = 0x20
DeviceEnabled: Final = 0x40
DeviceDisabled: Final = 0x80
AddMaster: Final = 1
RemoveMaster: Final = 2
AttachSlave: Final = 3
DetachSlave: Final = 4
AttachToMaster: Final = 1
Floating: Final = 2
ModeRelative: Final = 0
ModeAbsolute: Final = 1
MasterPointer: Final = 1
MasterKeyboard: Final = 2
SlavePointer: Final = 3
SlaveKeyboard: Final = 4
FloatingSlave: Final = 5
KeyClass: Final = 0
ButtonClass: Final = 1
ValuatorClass: Final = 2
ScrollClass: Final = 3
TouchClass: Final = 8
KeyRepeat: Final = 0x10000
AllDevices: Final = 0
AllMasterDevices: Final = 1
DeviceChanged: Final = 1
KeyPress: Final = 2
KeyRelease: Final = 3
ButtonPress: Final = 4
ButtonRelease: Final = 5
Motion: Final = 6
Enter: Final = 7
Leave: Final = 8
FocusIn: Final = 9
FocusOut: Final = 10
HierarchyChanged: Final = 11
PropertyEvent: Final = 12
RawKeyPress: Final = 13
RawKeyRelease: Final = 14
RawButtonPress: Final = 15
RawButtonRelease: Final = 16
RawMotion: Final = 17
DeviceChangedMask: Final = 0x00002
KeyPressMask: Final = 0x00004
KeyReleaseMask: Final = 0x00008
ButtonPressMask: Final = 0x00010
ButtonReleaseMask: Final = 0x00020
MotionMask: Final = 0x00040
EnterMask: Final = 0x00080
LeaveMask: Final = 0x00100
FocusInMask: Final = 0x00200
FocusOutMask: Final = 0x00400
HierarchyChangedMask: Final = 0x00800
PropertyEventMask: Final = 0x01000
RawKeyPressMask: Final = 0x02000
RawKeyReleaseMask: Final = 0x04000
RawButtonPressMask: Final = 0x08000
RawButtonReleaseMask: Final = 0x10000
RawMotionMask: Final = 0x20000
GrabModeSync: Final = 0
GrabModeAsync: Final = 1
GrabModeTouch: Final = 2
DEVICEID = rq.Card16
DEVICE = rq.Card16
DEVICEUSE = rq.Card8
PROPERTY_TYPE_FLOAT: Final = "FLOAT"

# ignore[override] because of Liskov substitution principle violations
class FP1616(rq.Int32):
    def check_value(self, value: float) -> int: ...  # type: ignore[override]
    def parse_value(self, value: ConvertibleToFloat, display: Unused) -> float: ...  # type: ignore[override]

class FP3232(rq.ValueField):
    structcode: str
    def check_value(self, value: _T) -> _T: ...  # type: ignore[override]
    def parse_value(self, value: tuple[ConvertibleToFloat, ConvertibleToFloat], display: Unused) -> float: ...  # type: ignore[override]

class XIQueryVersion(rq.ReplyRequest): ...

def query_version(self: Display | resource.Resource) -> XIQueryVersion: ...

class Mask(rq.List):
    def __init__(self, name: str) -> None: ...
    def pack_value(self, val: int | Iterable[int]) -> tuple[bytes, int, None]: ...  # type: ignore[override]

EventMask: rq.Struct

class XISelectEvents(rq.Request): ...

def select_events(self: drawable.Window, event_masks: Sequence[tuple[int, Sequence[int]]]) -> XISelectEvents: ...

AnyInfo: rq.Struct

class ButtonMask:
    def __init__(self, value: int, length: int) -> None: ...
    def __getitem__(self, key: int) -> int: ...
    def __len__(self) -> int: ...

class ButtonState(rq.ValueField):
    structcode: None
    def __init__(self, name: str) -> None: ...
    def parse_binary_value(  # type: ignore[override]  # length: None will error. See: https://github.com/python-xlib/python-xlib/pull/248
        self, data: SliceableBuffer, display: Unused, length: int, fmt: Unused
    ) -> tuple[ButtonMask, SliceableBuffer]: ...

ButtonInfo: rq.Struct
KeyInfo: rq.Struct
ValuatorInfo: rq.Struct
ScrollInfo: rq.Struct
TouchInfo: rq.Struct
INFO_CLASSES: Final[dict[int, rq.Struct]]

class ClassInfoClass:
    structcode: None
    def parse_binary(self, data: SliceableBuffer, display: display.Display | None) -> tuple[rq.DictWrapper, SliceableBuffer]: ...

ClassInfo: ClassInfoClass
DeviceInfo: rq.Struct

class XIQueryDevice(rq.ReplyRequest): ...

def query_device(self: Display | resource.Resource, deviceid: int) -> XIQueryDevice: ...

class XIListProperties(rq.ReplyRequest): ...

def list_device_properties(self: Display | resource.Resource, deviceid: int) -> XIListProperties: ...

class XIGetProperty(rq.ReplyRequest): ...

def get_device_property(
    self: Display | resource.Resource, deviceid: int, property: int, type: int, offset: int, length: int, delete: bool = False
) -> XIGetProperty: ...

class XIChangeProperty(rq.Request): ...

def change_device_property(
    self: Display | resource.Resource, deviceid: int, property: int, type: int, mode: int, value: Sequence[float] | Sequence[str]
) -> XIChangeProperty: ...

class XIDeleteProperty(rq.Request): ...

def delete_device_property(self: Display | resource.Resource, deviceid: int, property: int) -> XIDeleteProperty: ...

class XIGrabDevice(rq.ReplyRequest): ...

def grab_device(
    self: drawable.Window,
    deviceid: int,
    time: int,
    grab_mode: int,
    paired_device_mode: int,
    owner_events: bool,
    event_mask: Sequence[int],
) -> XIGrabDevice: ...

class XIUngrabDevice(rq.Request): ...

def ungrab_device(self: Display | resource.Resource, deviceid: int, time: int) -> XIUngrabDevice: ...

class XIPassiveGrabDevice(rq.ReplyRequest): ...

def passive_grab_device(
    self: drawable.Window,
    deviceid: int,
    time: int,
    detail: int,
    grab_type: int,
    grab_mode: int,
    paired_device_mode: int,
    owner_events: bool,
    event_mask: Sequence[int],
    modifiers: Sequence[int],
) -> XIPassiveGrabDevice: ...
def grab_keycode(
    self: drawable.Window,
    deviceid: int,
    time: int,
    keycode: int,
    grab_mode: int,
    paired_device_mode: int,
    owner_events: bool,
    event_mask: Sequence[int],
    modifiers: Sequence[int],
) -> XIPassiveGrabDevice: ...

class XIPassiveUngrabDevice(rq.Request): ...

def passive_ungrab_device(
    self: drawable.Window, deviceid: int, detail: int, grab_type: int, modifiers: Sequence[int]
) -> XIPassiveUngrabDevice: ...
def ungrab_keycode(self: drawable.Window, deviceid: int, keycode: int, modifiers: Sequence[int]) -> XIPassiveUngrabDevice: ...

HierarchyInfo: rq.Struct
HierarchyEventData: rq.Struct
ModifierInfo: rq.Struct
GroupInfo: rq.Struct
DeviceEventData: rq.Struct
DeviceChangedEventData: rq.Struct
PropertyEventData: rq.Struct

def init(disp: Display, info: request.QueryExtension) -> None: ...



from _typeshed import Incomplete
from collections.abc import Callable, Iterable, Sequence
from re import Match, Pattern
from types import ModuleType, SimpleNamespace as Struct
from typing import Any, ClassVar, Final, NoReturn
from typing_extensions import TypeAlias

from docutils import ApplicationError, DataError, nodes
from docutils.nodes import Node, system_message
from docutils.parsers.rst.languages import _RstLanguageModule
from docutils.statemachine import StateMachine, StateMachineWS, StateWS, StringList
from docutils.utils import Reporter

__docformat__: Final = "reStructuredText"

class MarkupError(DataError): ...
class UnknownInterpretedRoleError(DataError): ...
class InterpretedRoleNotImplementedError(DataError): ...
class ParserError(ApplicationError): ...
class MarkupMismatch(Exception): ...

class RSTStateMachine(StateMachineWS[list[str]]):
    language: _RstLanguageModule
    match_titles: bool
    memo: Struct | None
    document: nodes.document
    reporter: Reporter
    node: nodes.document | None
    section_level_offset: int
    def run(  # type: ignore[override]
        self,
        input_lines: Sequence[str] | StringList,
        document: nodes.document,
        input_offset: int = 0,
        match_titles: bool = True,
        inliner: Inliner | None = None,
    ) -> None: ...

class NestedStateMachine(RSTStateMachine):
    parent_state_machine: Incomplete | None
    def __init__(self, state_classes, initial_state, debug: bool = False, parent_state_machine=None) -> None: ...
    def run(  # type: ignore[override]
        self, input_lines: Sequence[str] | StringList, input_offset: int, memo, node, match_titles: bool = True
    ) -> list[str]: ...

class RSTState(StateWS[list[str]]):
    nested_sm: type[NestedStateMachine]
    nested_sm_cache: list[StateMachine[Incomplete]]
    def __init__(self, state_machine, debug: bool = False) -> None: ...
    memo: Incomplete
    reporter: Reporter
    inliner: Inliner
    document: nodes.document
    parent: Incomplete
    def runtime_init(self) -> None: ...
    def goto_line(self, abs_line_offset: int) -> None: ...
    def no_match(self, context: list[str], transitions): ...
    def bof(self, context: list[str]): ...
    def nested_parse(
        self,
        block: StringList,
        input_offset: int,
        node: nodes.Element | None = None,
        match_titles: bool = False,
        state_machine_class: StateMachineWS[Incomplete] | None = None,
        state_machine_kwargs: dict[Incomplete, Incomplete] | None = None,
    ) -> int: ...
    def nested_list_parse(
        self,
        block,
        input_offset: int,
        node,
        initial_state,
        blank_finish,
        blank_finish_state=None,
        extra_settings={},
        match_titles: bool = False,
        state_machine_class=None,
        state_machine_kwargs=None,
    ): ...
    def section(self, title: str, source, style, lineno: int, messages) -> None: ...
    def check_subsection(self, source, style, lineno: int): ...
    def title_inconsistent(self, sourcetext: str, lineno: int): ...
    def new_subsection(self, title: str, lineno: int, messages) -> None: ...
    def paragraph(self, lines: Iterable[str], lineno: int): ...
    def inline_text(self, text: str, lineno: int) -> tuple[list[Node], list[system_message]]: ...
    def unindent_warning(self, node_name: str): ...

def build_regexp(definition, compile_patterns: bool | None = True): ...

_BasicDefinition: TypeAlias = tuple[str, str, str, list[Pattern[str]]]
_DefinitionParts: TypeAlias = tuple[str, str, str, list[Pattern[str] | _BasicDefinition]]
_DefinitionType: TypeAlias = tuple[str, str, str, list[Pattern[str] | _DefinitionParts]]

class Inliner:
    implicit_dispatch: list[tuple[Pattern[str], Callable[[Match[str], int], Sequence[nodes.Node]]]]
    def __init__(self) -> None: ...
    start_string_prefix: str
    end_string_suffix: str
    parts: _DefinitionType
    patterns: Struct
    def init_customizations(self, settings: Any) -> None: ...
    reporter: Reporter
    document: nodes.document
    language: ModuleType
    parent: nodes.Element
    def parse(
        self, text: str, lineno: int, memo: Struct, parent: nodes.Element
    ) -> tuple[list[nodes.Node], list[nodes.system_message]]: ...
    non_whitespace_before: str
    non_whitespace_escape_before: str
    non_unescaped_whitespace_escape_before: str
    non_whitespace_after: str
    simplename: str
    uric: str
    uri_end_delim: str
    urilast: str
    uri_end: str
    emailc: str
    email_pattern: str
    def quoted_start(self, match: Match[str]) -> bool: ...
    def inline_obj(
        self,
        match: Match[str],
        lineno: int,
        end_pattern: Pattern[str],
        nodeclass: nodes.TextElement,
        restore_backslashes: bool = False,
    ) -> tuple[str, list[nodes.problematic], str, list[nodes.system_message], str]: ...
    def problematic(self, text: str, rawsource: str, message: nodes.system_message) -> nodes.problematic: ...
    def emphasis(
        self, match: Match[str], lineno: int
    ) -> tuple[str, list[nodes.problematic], str, list[nodes.system_message]]: ...
    def strong(self, match: Match[str], lineno: int) -> tuple[str, list[nodes.problematic], str, list[nodes.system_message]]: ...
    def interpreted_or_phrase_ref(
        self, match: Match[str], lineno: int
    ) -> tuple[str, list[nodes.problematic], str, list[nodes.system_message]]: ...
    def phrase_ref(
        self, before: str, after: str, rawsource: str, escaped: str, text: str | None = None
    ) -> tuple[str, list[nodes.Node], str, list[nodes.Node]]: ...
    def adjust_uri(self, uri: str) -> str: ...
    def interpreted(
        self, rawsource: str, text: str, role: str, lineno: int
    ) -> tuple[list[nodes.Node], list[nodes.system_message]]: ...
    def literal(self, match: Match[str], lineno: int) -> tuple[str, list[nodes.problematic], str, list[nodes.system_message]]: ...
    def inline_internal_target(
        self, match: Match[str], lineno: int
    ) -> tuple[str, list[nodes.problematic], str, list[nodes.system_message]]: ...
    def substitution_reference(
        self, match: Match[str], lineno: int
    ) -> tuple[str, list[nodes.problematic], str, list[nodes.system_message]]: ...
    def footnote_reference(
        self, match: Match[str], lineno: int
    ) -> tuple[str, list[nodes.problematic], str, list[nodes.system_message]]: ...
    def reference(
        self, match: Match[str], lineno: int, anonymous: bool = False
    ) -> tuple[str, list[nodes.problematic], str, list[nodes.system_message]]: ...
    def anonymous_reference(
        self, match: Match[str], lineno: int
    ) -> tuple[str, list[nodes.problematic], str, list[nodes.system_message]]: ...
    def standalone_uri(
        self, match: Match[str], lineno: int
    ) -> list[tuple[str, list[nodes.problematic], str, list[nodes.system_message]]]: ...
    def pep_reference(
        self, match: Match[str], lineno: int
    ) -> list[tuple[str, list[nodes.problematic], str, list[nodes.system_message]]]: ...
    rfc_url: str = ...
    def rfc_reference(
        self, match: Match[str], lineno: int
    ) -> list[tuple[str, list[nodes.problematic], str, list[nodes.system_message]]]: ...
    def implicit_inline(self, text: str, lineno: int) -> list[nodes.Text]: ...
    dispatch: dict[str, Callable[[Match[str], int], tuple[str, list[nodes.problematic], str, list[nodes.system_message]]]] = ...

class Body(RSTState):
    double_width_pad_char: Incomplete
    enum: Incomplete
    grid_table_top_pat: Incomplete
    simple_table_top_pat: Incomplete
    simple_table_border_pat: Incomplete
    pats: Incomplete
    patterns: ClassVar[dict[str, str | Pattern[str]]]
    initial_transitions: ClassVar[tuple[str, ...]]
    sequence: str
    format: str
    def indent(self, match, context, next_state): ...
    def block_quote(self, indented, line_offset): ...
    attribution_pattern: Incomplete
    def split_attribution(self, indented, line_offset): ...
    def check_attribution(self, indented, attribution_start): ...
    def parse_attribution(self, indented, line_offset): ...
    def bullet(self, match, context, next_state): ...
    def list_item(self, indent): ...
    def enumerator(self, match, context, next_state): ...
    def parse_enumerator(self, match, expected_sequence=None): ...
    def is_enumerated_list_item(self, ordinal, sequence, format): ...
    def make_enumerator(self, ordinal, sequence, format): ...
    def field_marker(self, match, context, next_state): ...
    def field(self, match): ...
    def parse_field_marker(self, match): ...
    def parse_field_body(self, indented, offset, node) -> None: ...
    def option_marker(self, match, context, next_state): ...
    def option_list_item(self, match): ...
    def parse_option_marker(self, match): ...
    def doctest(self, match, context, next_state): ...
    def line_block(self, match, context, next_state): ...
    def line_block_line(self, match, lineno): ...
    def nest_line_block_lines(self, block) -> None: ...
    def nest_line_block_segment(self, block) -> None: ...
    def grid_table_top(self, match, context, next_state): ...
    def simple_table_top(self, match, context, next_state): ...
    def table_top(self, match, context, next_state, isolate_function, parser_class): ...
    def table(self, isolate_function, parser_class): ...
    def isolate_grid_table(self): ...
    def isolate_simple_table(self): ...
    def malformed_table(self, block, detail: str = "", offset: int = 0): ...
    def build_table(self, tabledata, tableline, stub_columns: int = 0, widths=None) -> nodes.table: ...
    def build_table_row(self, rowdata, tableline): ...
    explicit: Incomplete
    def footnote(self, match): ...
    def citation(self, match): ...
    def hyperlink_target(self, match): ...
    def make_target(self, block, block_text, lineno, target_name): ...
    def parse_target(self, block, block_text, lineno): ...
    def is_reference(self, reference): ...
    def add_target(self, targetname, refuri, target, lineno) -> None: ...
    def substitution_def(self, match): ...
    def disallowed_inside_substitution_definitions(self, node): ...
    def directive(self, match, **option_presets): ...
    def run_directive(self, directive, match, type_name, option_presets): ...
    def parse_directive_block(self, indented, line_offset, directive, option_presets): ...
    def parse_directive_options(self, option_presets, option_spec, arg_block): ...
    def parse_directive_arguments(self, directive, arg_block): ...
    def parse_extension_options(self, option_spec, datalines): ...
    def unknown_directive(self, type_name): ...
    def comment(self, match): ...
    def explicit_markup(self, match, context, next_state): ...
    def explicit_construct(self, match): ...
    def explicit_list(self, blank_finish) -> None: ...
    def anonymous(self, match: Match[str], context: list[str] | None, next_state: str): ...
    def anonymous_target(self, match): ...
    def line(self, match, context, next_state): ...
    def text(self, match, context, next_state): ...

class RFC2822Body(Body):
    patterns: ClassVar[dict[str, str | Pattern[str]]]
    initial_transitions: ClassVar[list[tuple[str | tuple[str, str], str]]]  # type: ignore[assignment]
    def rfc2822(self, match, context, next_state): ...
    def rfc2822_field(self, match): ...

class SpecializedBody(Body):
    def invalid_input(
        self, match: Match[str] | None = None, context: list[str] | None = None, next_state: str | None = None
    ) -> NoReturn: ...
    indent = invalid_input  # type: ignore[assignment]
    bullet = invalid_input
    enumerator = invalid_input
    field_marker = invalid_input
    option_marker = invalid_input
    doctest = invalid_input
    line_block = invalid_input
    grid_table_top = invalid_input
    simple_table_top = invalid_input
    explicit_markup = invalid_input
    anonymous = invalid_input  # type: ignore[assignment]
    line = invalid_input
    text = invalid_input

class BulletList(SpecializedBody):
    blank_finish: Incomplete
    def bullet(  # type: ignore[override]
        self, match: Match[str], context: list[str] | None, next_state: str | None
    ) -> tuple[list[str], str | None, list[str]]: ...

class DefinitionList(SpecializedBody):
    def text(self, match: Match[str], context: list[str] | None, next_state: str | None) -> tuple[list[str], str, list[str]]: ...  # type: ignore[override]

class EnumeratedList(SpecializedBody):
    auto: int
    blank_finish: Incomplete
    lastordinal: Incomplete
    def enumerator(  # type: ignore[override]
        self, match: Match[str], context: list[str] | None, next_state: str | None
    ) -> tuple[list[str], str | None, list[str]]: ...

class FieldList(SpecializedBody):
    blank_finish: Incomplete
    def field_marker(  # type: ignore[override]
        self, match: Match[str], context: list[str] | None, next_state: str | None
    ) -> tuple[list[str], str | None, list[str]]: ...

class OptionList(SpecializedBody):
    blank_finish: Incomplete
    def option_marker(  # type: ignore[override]
        self, match: Match[str], context: list[str] | None, next_state: str | None
    ) -> tuple[list[str], str | None, list[str]]: ...

class RFC2822List(SpecializedBody, RFC2822Body):
    patterns: ClassVar[dict[str, str | Pattern[str]]]
    initial_transitions: ClassVar[list[tuple[str | tuple[str, str], str]]]  # type: ignore[assignment]
    blank_finish: Incomplete
    def rfc2822(self, match, context, next_state): ...
    blank: Incomplete

class ExtensionOptions(FieldList):
    def parse_field_body(self, indented, offset, node) -> None: ...

class LineBlock(SpecializedBody):
    blank: Incomplete
    blank_finish: Incomplete
    def line_block(  # type: ignore[override]
        self, match: Match[str], context: list[str] | None, next_state: str | None
    ) -> tuple[list[str], str | None, list[str]]: ...

class Explicit(SpecializedBody):
    blank_finish: Incomplete
    blank: Incomplete
    def explicit_markup(  # type: ignore[override]
        self, match: Match[str], context: list[str] | None, next_state: str | None
    ) -> tuple[list[str], str | None, list[str]]: ...
    def anonymous(  # type: ignore[override]
        self, match: Match[str], context: list[str] | None, next_state: str | None
    ) -> tuple[list[str], str | None, list[str]]: ...

class SubstitutionDef(Body):
    patterns: ClassVar[dict[str, str | Pattern[str]]]
    initial_transitions: ClassVar[list[str]]  # type: ignore[assignment]
    blank_finish: Incomplete
    def embedded_directive(self, match, context, next_state) -> None: ...
    def text(self, match, context, next_state) -> None: ...

class Text(RSTState):
    patterns: ClassVar[dict[str, str | Pattern[str]]]
    initial_transitions: ClassVar[list[tuple[str, str]]]
    def blank(self, match, context, next_state): ...
    def eof(self, context): ...
    def indent(self, match, context, next_state): ...
    def underline(self, match, context, next_state): ...
    def text(self, match, context, next_state): ...
    def literal_block(self): ...
    def quoted_literal_block(self): ...
    def definition_list_item(self, termline): ...
    classifier_delimiter: Incomplete
    def term(self, lines, lineno): ...

class SpecializedText(Text):
    def eof(self, context): ...
    def invalid_input(
        self, match: Match[str] | None = None, context: list[str] | None = None, next_state: str | None = None
    ) -> NoReturn: ...
    blank = invalid_input
    indent = invalid_input
    underline = invalid_input
    text = invalid_input

class Definition(SpecializedText):
    def eof(self, context): ...
    blank_finish: Incomplete
    def indent(  # type: ignore[override]
        self, match: Match[str] | None, context: list[str], next_state: str | None
    ) -> tuple[list[str], str, list[str]]: ...

class Line(SpecializedText):
    eofcheck: int
    def eof(self, context: list[str]): ...
    def blank(self, match: Match[str] | None, context: list[str], next_state: str | None) -> tuple[list[str], str, list[str]]: ...  # type: ignore[override]
    def text(self, match: Match[str], context: list[str], next_state: str | None) -> tuple[list[str], str, list[str]]: ...  # type: ignore[override]
    indent = text  # type: ignore[assignment]
    def underline(  # type: ignore[override]
        self, match: Match[str] | None, context: list[str], next_state: str | None
    ) -> tuple[list[str], str, list[str]]: ...
    def short_overline(self, context, blocktext, lineno, lines: int = 1) -> None: ...
    def state_correction(self, context, lines: int = 1) -> None: ...

class QuotedLiteralBlock(RSTState):
    patterns: ClassVar[dict[str, str | Pattern[str]]]
    messages: Incomplete
    initial_lineno: Incomplete
    def __init__(self, state_machine, debug: bool = False) -> None: ...
    def blank(self, match, context, next_state): ...
    def eof(self, context): ...
    def indent(self, match: Match[str] | None, context: list[str], next_state: str | None) -> NoReturn: ...
    def initial_quoted(
        self, match: Match[str], context: list[str] | None, next_state: str | None
    ) -> tuple[list[str], str | None, list[str]]: ...
    def quoted(
        self, match: Match[str], context: list[str], next_state: str | None
    ) -> tuple[list[str], str | None, list[str]]: ...
    def text(self, match: Match[str] | None, context: list[str] | None, next_state: str | None) -> None: ...

state_classes: tuple[type[RSTState], ...]



import abc
from typing import Any, Generic, TypeVar, overload
from typing_extensions import ParamSpec

import tensorflow as tf
from tensorflow._aliases import ContainerGeneric

_P = ParamSpec("_P")
_R_co = TypeVar("_R_co", covariant=True)

class Callable(Generic[_P, _R_co], metaclass=abc.ABCMeta):
    def __call__(self, *args: _P.args, **kwargs: _P.kwargs) -> _R_co: ...

class ConcreteFunction(Callable[_P, _R_co], metaclass=abc.ABCMeta):
    def __call__(self, *args: _P.args, **kwargs: _P.kwargs) -> _R_co: ...

class PolymorphicFunction(Callable[_P, _R_co], metaclass=abc.ABCMeta):
    @overload
    @abc.abstractmethod
    def get_concrete_function(self, *args: _P.args, **kwargs: _P.kwargs) -> ConcreteFunction[_P, _R_co]: ...
    @overload
    @abc.abstractmethod
    def get_concrete_function(
        self, *args: ContainerGeneric[tf.TypeSpec[Any]], **kwargs: ContainerGeneric[tf.TypeSpec[Any]]
    ) -> ConcreteFunction[_P, _R_co]: ...
    def experimental_get_compiler_ir(self, *args, **kwargs): ...

GenericFunction = PolymorphicFunction

def __getattr__(name: str): ...  # incomplete module



from typing import Final, Literal

EPOCH: Final = 1825029.5
MONTHS: Final[list[str]]
WEEKDAYS: Final = ["Tkyriaka", "Pesnau", "Pshoment", "Peftoou", "Ptiou", "Psoou", "Psabbaton"]

def is_leap(year: int) -> bool: ...
def to_jd(year: int, month: int, day: int) -> float: ...
def from_jd(jdc: float) -> tuple[int, int, int]: ...
def to_gregorian(year: int, month: int, day: int) -> tuple[int, int, int]: ...
def from_gregorian(year: int, month: int, day: int) -> tuple[int, int, int]: ...
def month_length(year: int, month: int) -> Literal[5, 6, 30]: ...
def monthcalendar(year: int, month: int) -> list[list[int | None]]: ...
def format(year: int, month: int, day: int) -> str: ...



from collections.abc import Generator
from typing import Literal

from networkx.classes.digraph import DiGraph
from networkx.classes.graph import Graph, _NBunch, _Node
from networkx.utils.backends import _dispatchable

__all__ = [
    "cycle_basis",
    "simple_cycles",
    "recursive_simple_cycles",
    "find_cycle",
    "minimum_cycle_basis",
    "chordless_cycles",
    "girth",
]

@_dispatchable
def cycle_basis(G: Graph[_Node], root: _Node | None = None) -> list[list[_Node]]: ...
@_dispatchable
def simple_cycles(G: Graph[_Node], length_bound: int | None = None) -> Generator[list[_Node]]: ...
@_dispatchable
def chordless_cycles(G: DiGraph[_Node], length_bound: int | None = None) -> Generator[list[_Node]]: ...
@_dispatchable
def recursive_simple_cycles(G: DiGraph[_Node]) -> list[list[_Node]]: ...
@_dispatchable
def find_cycle(
    G: Graph[_Node], source: _NBunch[_Node] = None, orientation: Literal["original", "reverse", "ignore"] | None = None
): ...
@_dispatchable
def minimum_cycle_basis(G: Graph[_Node], weight: str | None = None) -> list[list[_Node]]: ...
@_dispatchable
def girth(G: Graph[_Node]) -> float | int: ...  # accepts any graph type



from typing import Final

class CffiOp:
    op: int | None
    arg: str | None
    def __init__(self, op: int | None, arg: str | None) -> None: ...
    def as_c_expr(self) -> str: ...
    def as_python_bytes(self) -> str: ...

def format_four_bytes(num: int) -> str: ...

OP_PRIMITIVE: Final = 1
OP_POINTER: Final = 3
OP_ARRAY: Final = 5
OP_OPEN_ARRAY: Final = 7
OP_STRUCT_UNION: Final = 9
OP_ENUM: Final = 11
OP_FUNCTION: Final = 13
OP_FUNCTION_END: Final = 15
OP_NOOP: Final = 17
OP_BITFIELD: Final = 19
OP_TYPENAME: Final = 21
OP_CPYTHON_BLTN_V: Final = 23
OP_CPYTHON_BLTN_N: Final = 25
OP_CPYTHON_BLTN_O: Final = 27
OP_CONSTANT: Final = 29
OP_CONSTANT_INT: Final = 31
OP_GLOBAL_VAR: Final = 33
OP_DLOPEN_FUNC: Final = 35
OP_DLOPEN_CONST: Final = 37
OP_GLOBAL_VAR_F: Final = 39
OP_EXTERN_PYTHON: Final = 41
PRIM_VOID: Final = 0
PRIM_BOOL: Final = 1
PRIM_CHAR: Final = 2
PRIM_SCHAR: Final = 3
PRIM_UCHAR: Final = 4
PRIM_SHORT: Final = 5
PRIM_USHORT: Final = 6
PRIM_INT: Final = 7
PRIM_UINT: Final = 8
PRIM_LONG: Final = 9
PRIM_ULONG: Final = 10
PRIM_LONGLONG: Final = 11
PRIM_ULONGLONG: Final = 12
PRIM_FLOAT: Final = 13
PRIM_DOUBLE: Final = 14
PRIM_LONGDOUBLE: Final = 15
PRIM_WCHAR: Final = 16
PRIM_INT8: Final = 17
PRIM_UINT8: Final = 18
PRIM_INT16: Final = 19
PRIM_UINT16: Final = 20
PRIM_INT32: Final = 21
PRIM_UINT32: Final = 22
PRIM_INT64: Final = 23
PRIM_UINT64: Final = 24
PRIM_INTPTR: Final = 25
PRIM_UINTPTR: Final = 26
PRIM_PTRDIFF: Final = 27
PRIM_SIZE: Final = 28
PRIM_SSIZE: Final = 29
PRIM_INT_LEAST8: Final = 30
PRIM_UINT_LEAST8: Final = 31
PRIM_INT_LEAST16: Final = 32
PRIM_UINT_LEAST16: Final = 33
PRIM_INT_LEAST32: Final = 34
PRIM_UINT_LEAST32: Final = 35
PRIM_INT_LEAST64: Final = 36
PRIM_UINT_LEAST64: Final = 37
PRIM_INT_FAST8: Final = 38
PRIM_UINT_FAST8: Final = 39
PRIM_INT_FAST16: Final = 40
PRIM_UINT_FAST16: Final = 41
PRIM_INT_FAST32: Final = 42
PRIM_UINT_FAST32: Final = 43
PRIM_INT_FAST64: Final = 44
PRIM_UINT_FAST64: Final = 45
PRIM_INTMAX: Final = 46
PRIM_UINTMAX: Final = 47
PRIM_FLOATCOMPLEX: Final = 48
PRIM_DOUBLECOMPLEX: Final = 49
PRIM_CHAR16: Final = 50
PRIM_CHAR32: Final = 51
PRIMITIVE_TO_INDEX: Final[dict[str, int]]
F_UNION: Final = 1
F_CHECK_FIELDS: Final = 2
F_PACKED: Final = 4
F_EXTERNAL: Final = 8
F_OPAQUE: Final = 16
G_FLAGS: Final[dict[bytes, bytes]]
CLASS_NAME: Final[dict[int, str]]



# Internal-use module for types shared by multiple modules.
# This does not match a module in docker-py.

from pathlib import Path
from typing import TypedDict, type_check_only
from typing_extensions import NotRequired, TypeAlias

# Type alias for JSON, explained at:
# https://github.com/python/typing/issues/182#issuecomment-1320974824.
JSON: TypeAlias = dict[str, JSON] | list[JSON] | str | int | float | bool | None

@type_check_only
class ContainerWeightDevice(TypedDict):
    Path: Path
    Weight: int

# See https://docs.docker.com/engine/api/v1.42/#tag/Container/operation/ContainerWait
@type_check_only
class _WaitErrorDetails(TypedDict):
    Message: str

@type_check_only
class WaitContainerResponse(TypedDict):
    StatusCode: int
    Error: NotRequired[_WaitErrorDetails]



from _typeshed import Incomplete
from collections.abc import Iterable
from logging import Logger

from .base import BaseEndpoint

log: Logger

class MetadataEndpoint(BaseEndpoint):
    raise_errors: bool
    endpoints: Iterable[BaseEndpoint]
    initial_claims: dict[str, Incomplete]
    claims: dict[str, Incomplete]
    def __init__(
        self, endpoints: Iterable[BaseEndpoint], claims: dict[str, Incomplete] = {}, raise_errors: bool = True
    ) -> None: ...
    def create_metadata_response(
        self, uri: str, http_method: str = "GET", body: str | None = None, headers: dict[str, str] | None = None
    ) -> tuple[dict[str, str], str, int]: ...
    def validate_metadata(
        self, array, key, is_required: bool = False, is_list: bool = False, is_url: bool = False, is_issuer: bool = False
    ) -> None: ...
    def validate_metadata_token(self, claims, endpoint: BaseEndpoint) -> None: ...
    def validate_metadata_authorization(self, claims, endpoint: BaseEndpoint): ...
    def validate_metadata_revocation(self, claims, endpoint: BaseEndpoint) -> None: ...
    def validate_metadata_introspection(self, claims, endpoint: BaseEndpoint) -> None: ...
    def validate_metadata_server(self) -> dict[str, Incomplete]: ...



from _typeshed import Incomplete
from typing import ClassVar

from setuptools.dist import Distribution

from .._distutils.command.build_ext import build_ext as _build_ext

have_rtld: bool
use_stubs: bool
libtype: str

def get_abi3_suffix(): ...

class build_ext(_build_ext):
    distribution: Distribution  # override distutils.dist.Distribution with setuptools.dist.Distribution
    editable_mode: ClassVar[bool]
    inplace: bool
    def run(self) -> None: ...
    def copy_extensions_to_source(self) -> None: ...
    def get_ext_filename(self, fullname): ...
    shlib_compiler: Incomplete
    shlibs: list[Incomplete]
    ext_map: dict[Incomplete, Incomplete]
    def initialize_options(self) -> None: ...
    extensions: list[Incomplete]
    def finalize_options(self) -> None: ...
    def setup_shlib_compiler(self) -> None: ...
    def get_export_symbols(self, ext): ...
    compiler: Incomplete
    def build_extension(self, ext) -> None: ...
    def links_to_dynamic(self, ext): ...
    def get_source_files(self) -> list[str]: ...
    def get_outputs(self) -> list[str]: ...
    def get_output_mapping(self) -> dict[str, str]: ...
    def write_stub(self, output_dir, ext, compile: bool = False) -> None: ...

def link_shared_object(
    self,
    objects,
    output_libname,
    output_dir=None,
    libraries=None,
    library_dirs=None,
    runtime_library_dirs=None,
    export_symbols=None,
    debug: bool = False,
    extra_preargs=None,
    extra_postargs=None,
    build_temp=None,
    target_lang=None,
) -> None: ...



import abc
from collections.abc import Callable, Generator
from contextlib import AbstractContextManager, contextmanager
from typing import Literal
from typing_extensions import Self

import tensorflow as tf
from tensorflow._aliases import FloatArray, IntArray
from tensorflow.core.framework.graph_pb2 import GraphDef
from tensorflow.experimental.dtensor import Mesh

class SummaryWriter(metaclass=abc.ABCMeta):
    def as_default(self, step: int | None = None) -> AbstractContextManager[Self]: ...
    def close(self) -> None: ...
    def flush(self) -> None: ...
    def init(self) -> None: ...
    def set_as_default(self, step: int | None = None) -> None: ...

def audio(
    name: str,
    data: tf.Tensor,
    sample_rate: int | tf.Tensor,
    step: int | tf.Tensor | None = None,
    max_outputs: int | tf.Tensor | None = 3,
    encoding: Literal["wav"] | None = None,
    description: str | None = None,
) -> bool: ...
def create_file_writer(
    logdir: str,
    max_queue: int | None = None,
    flush_millis: int | None = None,
    filename_suffix: str | None = None,
    name: str | None = None,
    experimental_trackable: bool = False,
    experimental_mesh: Mesh | None = None,
) -> SummaryWriter: ...
def create_noop_writer() -> SummaryWriter: ...
def flush(writer: SummaryWriter | None = None, name: str | None = None) -> tf.Operation: ...
def graph(graph_data: tf.Graph | GraphDef) -> bool: ...
def histogram(
    name: str, data: tf.Tensor, step: int | None = None, buckets: int | None = None, description: str | None = None
) -> bool: ...
def image(
    name: str,
    data: tf.Tensor | FloatArray | IntArray,
    step: int | tf.Tensor | None = None,
    max_outputs: int | None = 3,
    description: str | None = None,
) -> bool: ...
@contextmanager
def record_if(condition: bool | tf.Tensor | Callable[[], bool]) -> Generator[None]: ...
def scalar(name: str, data: float | tf.Tensor, step: int | tf.Tensor | None = None, description: str | None = None) -> bool: ...
def should_record_summaries() -> bool: ...
def text(name: str, data: str | tf.Tensor, step: int | tf.Tensor | None = None, description: str | None = None) -> bool: ...
def trace_export(name: str, step: int | tf.Tensor | None = None, profiler_outdir: str | None = None) -> None: ...
def trace_off() -> None: ...
def trace_on(graph: bool = True, profiler: bool = False, profiler_outdir: str | None = None) -> None: ...
def write(tag: str, tensor: tf.Tensor, step: int | tf.Tensor | None = None, metadata=None, name: str | None = None) -> bool: ...



from collections.abc import Callable, Generator
from typing import TypeVar

from antlr4.error.Errors import RecognitionException
from antlr4.RuleContext import RuleContext as RuleContext
from antlr4.Token import Token as Token
from antlr4.tree.Tree import (
    INVALID_INTERVAL as INVALID_INTERVAL,
    ErrorNodeImpl as ErrorNodeImpl,
    ParseTree as ParseTree,
    ParseTreeListener as ParseTreeListener,
    TerminalNode as TerminalNode,
    TerminalNodeImpl as TerminalNodeImpl,
)

class ParserRuleContext(RuleContext):
    __slots__ = ("children", "start", "stop", "exception")
    children: list[ParseTree | TerminalNode] | None
    start: Token | None
    stop: Token | None
    exception: RecognitionException | None
    def __init__(self, parent: ParserRuleContext | None = None, invokingStateNumber: int | None = None) -> None: ...
    parentCtx: RuleContext | None
    invokingState: int
    def copyFrom(self, ctx: ParserRuleContext) -> None: ...
    def enterRule(self, listener: ParseTreeListener) -> None: ...
    def exitRule(self, listener: ParseTreeListener) -> None: ...
    def addChild(self, child: _ParseTreeT) -> _ParseTreeT: ...
    def removeLastChild(self) -> None: ...
    def addTokenNode(self, token: Token) -> TerminalNodeImpl: ...
    def addErrorNode(self, badToken: Token) -> ErrorNodeImpl: ...
    def getChild(self, i: int, ttype: type[_GenericType] | None = None) -> _GenericType | None: ...
    def getChildren(
        self, predicate: Callable[[ParseTree | TerminalNode], bool] | None = None
    ) -> Generator[ParseTree | TerminalNode, None, None]: ...
    def getToken(self, ttype: int, i: int) -> TerminalNode | None: ...
    def getTokens(self, ttype: int) -> list[TerminalNode]: ...
    def getTypedRuleContext(self, ctxType: type[_ParserRuleContextT], i: int) -> _ParserRuleContextT | None: ...
    def getTypedRuleContexts(self, ctxType: type[_ParserRuleContextT]) -> list[_ParserRuleContextT]: ...
    def getChildCount(self) -> int: ...
    def getSourceInterval(self) -> tuple[int | None, int | None]: ...

_GenericType = TypeVar("_GenericType", bound=type)
_ParseTreeT = TypeVar("_ParseTreeT", bound=ParseTree)
_ParserRuleContextT = TypeVar("_ParserRuleContextT", bound=ParserRuleContext)

class InterpreterRuleContext(ParserRuleContext):
    ruleIndex: int
    def __init__(self, parent: ParserRuleContext, invokingStateNumber: int, ruleIndex: int) -> None: ...



from _typeshed import Incomplete

from braintree.error_result import ErrorResult
from braintree.merchant_account import MerchantAccount
from braintree.successful_result import SuccessfulResult

class MerchantAccountGateway:
    gateway: Incomplete
    config: Incomplete
    def __init__(self, gateway) -> None: ...
    def find(self, merchant_account_id: str) -> MerchantAccount: ...
    def create_for_currency(self, params: dict[str, Incomplete] | None = None) -> SuccessfulResult | ErrorResult | None: ...
    def all(self) -> SuccessfulResult: ...



#!/usr/bin/env python3

"""Script to generate unannotated baseline stubs using stubgen.

Basic usage:
$ python3 scripts/create_baseline_stubs.py <project on PyPI>

Run with -h for more help.
"""

from __future__ import annotations

import argparse
import asyncio
import re
import subprocess
import sys
import urllib.parse
from http import HTTPStatus
from importlib.metadata import distribution
from pathlib import Path

import aiohttp
import termcolor

from ts_utils.paths import PYRIGHT_CONFIG, STDLIB_PATH, STUBS_PATH


def search_pip_freeze_output(project: str, output: str) -> tuple[str, str] | None:
    # Look for lines such as "typed-ast==1.4.2".  '-' matches '_' and
    # '_' matches '-' in project name, so that "typed_ast" matches
    # "typed-ast", and vice versa.
    regex = "^(" + re.sub(r"[-_]", "[-_]", project) + ")==(.*)"
    m = re.search(regex, output, flags=re.IGNORECASE | re.MULTILINE)
    if not m:
        return None
    return m.group(1), m.group(2)


def get_installed_package_info(project: str) -> tuple[str, str] | None:
    """Find package information from pip freeze output.

    Match project name somewhat fuzzily (case sensitive; '-' matches '_', and
    vice versa).

    Return (normalized project name, installed version) if successful.
    """
    # Not using "uv pip freeze" because if this is run from a global Python,
    # it'll mistakenly list the .venv's packages.
    r = subprocess.run(["pip", "freeze"], capture_output=True, text=True, check=True)
    return search_pip_freeze_output(project, r.stdout)


def run_stubgen(package: str, output: Path) -> None:
    print(f"Running stubgen: stubgen -o {output} -p {package}")
    subprocess.run(["stubgen", "-o", output, "-p", package, "--export-less"], check=True)


def run_stubdefaulter(stub_dir: Path) -> None:
    print(f"Running stubdefaulter: stubdefaulter --packages {stub_dir}")
    subprocess.run(["stubdefaulter", "--packages", stub_dir], check=False)


def run_black(stub_dir: Path) -> None:
    print(f"Running Black: black {stub_dir}")
    subprocess.run(["pre-commit", "run", "black", "--files", *stub_dir.rglob("*.pyi")], check=False)


def run_ruff(stub_dir: Path) -> None:
    print(f"Running Ruff: ruff check {stub_dir} --fix-only")
    subprocess.run([sys.executable, "-m", "ruff", "check", stub_dir, "--fix-only"], check=False)


async def get_project_urls_from_pypi(project: str, session: aiohttp.ClientSession) -> dict[str, str]:
    pypi_root = f"https://pypi.org/pypi/{urllib.parse.quote(project)}"
    async with session.get(f"{pypi_root}/json") as response:
        if response.status != HTTPStatus.OK:
            return {}
        j: dict[str, dict[str, dict[str, str]]]
        j = await response.json()
        return j["info"].get("project_urls") or {}


async def get_upstream_repo_url(project: str) -> str | None:
    # aiohttp is overkill here, but it would also just be silly
    # to have both requests and aiohttp in our requirements-tests.txt file.
    async with aiohttp.ClientSession() as session:
        project_urls = await get_project_urls_from_pypi(project, session)

        if not project_urls:
            return None

        # Order the project URLs so that we put the ones
        # that are most likely to point to the source code first
        urls_to_check: list[str] = []
        url_names_probably_pointing_to_source = ("Source", "Repository", "Homepage")
        for url_name in url_names_probably_pointing_to_source:
            if url := project_urls.get(url_name):
                urls_to_check.append(url)
        urls_to_check.extend(
            url for url_name, url in project_urls.items() if url_name not in url_names_probably_pointing_to_source
        )

        for url_to_check in urls_to_check:
            # Remove `www.`; replace `http://` with `https://`
            url = re.sub(r"^(https?://)?(www\.)?", "https://", url_to_check)
            netloc = urllib.parse.urlparse(url).netloc
            if netloc in {"gitlab.com", "github.com", "bitbucket.org", "foss.heptapod.net"}:
                # truncate to https://site.com/user/repo
                upstream_repo_url = "/".join(url.split("/")[:5])
                async with session.get(upstream_repo_url) as response:
                    if response.status == HTTPStatus.OK:
                        return upstream_repo_url
    return None


def create_metadata(project: str, stub_dir: Path, version: str) -> None:
    """Create a METADATA.toml file."""
    match = re.match(r"[0-9]+.[0-9]+", version)
    if match is None:
        sys.exit(f"Error: Cannot parse version number: {version}")
    filename = stub_dir / "METADATA.toml"
    version = match.group(0)
    if filename.exists():
        return
    metadata = f'version = "{version}.*"\n'
    upstream_repo_url = asyncio.run(get_upstream_repo_url(project))
    if upstream_repo_url is None:
        warning = (
            f"\nCould not find a URL pointing to the source code for {project!r}.\n"
            f"Please add it as `upstream_repository` to `stubs/{project}/METADATA.toml`, if possible!\n"
        )
        print(termcolor.colored(warning, "red"))
    else:
        metadata += f'upstream_repository = "{upstream_repo_url}"\n'
    print(f"Writing {filename}")
    filename.write_text(metadata, encoding="UTF-8")


def add_pyright_exclusion(stub_dir: Path) -> None:
    """Exclude stub_dir from strict pyright checks."""
    with PYRIGHT_CONFIG.open(encoding="UTF-8") as f:
        lines = f.readlines()
    i = 0
    while i < len(lines) and not lines[i].strip().startswith('"exclude": ['):
        i += 1
    assert i < len(lines), f"Error parsing {PYRIGHT_CONFIG}"
    while not lines[i].strip().startswith("]"):
        i += 1
    end = i

    # We assume that all third-party excludes must be at the end of the list.
    # This helps with skipping special entries, such as "stubs/**/@tests/test_cases".
    while lines[i - 1].strip().startswith('"stubs/'):
        i -= 1
    start = i

    before_third_party_excludes = lines[:start]
    third_party_excludes = lines[start:end]
    after_third_party_excludes = lines[end:]

    last_line = third_party_excludes[-1].rstrip()
    if not last_line.endswith(","):
        last_line += ","
        third_party_excludes[-1] = last_line + "\n"

    # Must use forward slash in the .json file
    line_to_add = f'        "{stub_dir.as_posix()}",\n'

    if line_to_add in third_party_excludes:
        print(f"{PYRIGHT_CONFIG} already up-to-date")
        return

    third_party_excludes.append(line_to_add)
    third_party_excludes.sort(key=str.lower)

    print(f"Updating {PYRIGHT_CONFIG}")
    with PYRIGHT_CONFIG.open("w", encoding="UTF-8") as f:
        f.writelines(before_third_party_excludes)
        f.writelines(third_party_excludes)
        f.writelines(after_third_party_excludes)


def main() -> None:
    parser = argparse.ArgumentParser(
        description="""Generate baseline stubs automatically for an installed pip package
                       using stubgen. Also run Black and Ruff. If the name of
                       the project is different from the runtime Python package name, you may
                       need to use --package (example: --package yaml PyYAML)."""
    )
    parser.add_argument("project", help="name of PyPI project for which to generate stubs under stubs/")
    parser.add_argument("--package", help="generate stubs for this Python package (default is autodetected)")
    args = parser.parse_args()
    project = args.project
    package: str = args.package

    if not re.match(r"[a-zA-Z0-9-_.]+$", project):
        sys.exit(f"Invalid character in project name: {project!r}")

    if not package:
        package = project  # default
        # Try to find which packages are provided by the project
        # Use default if that fails or if several packages are found
        #
        # The importlib.metadata module is used for projects whose name is different
        # from the runtime Python package name (example: PyYAML/yaml)
        dist = distribution(project).read_text("top_level.txt")
        if dist is not None:
            packages = [name for name in dist.split() if not name.startswith("_")]
            if len(packages) == 1:
                package = packages[0]
        print(f'Using detected package "{package}" for project "{project}"', file=sys.stderr)
        print("Suggestion: Try again with --package argument if that's not what you wanted", file=sys.stderr)

    if not STUBS_PATH.is_dir() or not STDLIB_PATH.is_dir():
        sys.exit("Error: Current working directory must be the root of typeshed repository")

    # Get normalized project name and version of installed package.
    info = get_installed_package_info(project)
    if info is None:
        print(f'Error: "{project}" is not installed', file=sys.stderr)
        print(file=sys.stderr)
        print(f"Suggestion: Run `{sys.executable} -m pip install {project}` and try again", file=sys.stderr)
        sys.exit(1)
    project, version = info

    stub_dir = STUBS_PATH / project
    package_dir = stub_dir / package
    if package_dir.exists():
        sys.exit(f"Error: {package_dir} already exists (delete it first)")

    run_stubgen(package, stub_dir)
    run_stubdefaulter(stub_dir)

    run_ruff(stub_dir)
    run_black(stub_dir)

    create_metadata(project, stub_dir, version)

    # Since the generated stubs won't have many type annotations, we
    # have to exclude them from strict pyright checks.
    add_pyright_exclusion(stub_dir)

    print("\nDone!\n\nSuggested next steps:")
    print(f" 1. Manually review the generated stubs in {stub_dir}")
    print(" 2. Optionally run tests and autofixes (see tests/README.md for details)")
    print(" 3. Commit the changes on a new branch and create a typeshed PR (don't force-push!)")


if __name__ == "__main__":
    main()



import optparse
from _typeshed import Incomplete, StrPath
from collections.abc import Iterable, Mapping, Sequence
from configparser import RawConfigParser
from typing import Any, ClassVar, Final, Literal, Protocol, overload, type_check_only
from typing_extensions import deprecated

from docutils import SettingsSpec
from docutils.utils import DependencyList

__docformat__: Final = "reStructuredText"

@type_check_only
class _OptionValidator(Protocol):
    def __call__(
        self,
        setting: str,
        value: str | None,
        option_parser: OptionParser,
        /,
        config_parser: ConfigParser | None = None,
        config_section: str | None = None,
    ) -> Any: ...

@deprecated("Deprecated and will be removed with the switch to from optparse to argparse.")
def store_multiple(option: optparse.Option, opt: str, value, parser: OptionParser, *args: str, **kwargs) -> None: ...
@deprecated("Deprecated and will be removed with the switch to from optparse to argparse.")
def read_config_file(option: optparse.Option, opt: str, value, parser: OptionParser) -> None: ...
def validate_encoding(
    setting: str,
    value: str | None = None,
    option_parser: OptionParser | None = None,
    config_parser: ConfigParser | None = None,
    config_section: str | None = None,
) -> str: ...
def validate_encoding_error_handler(
    setting: str,
    value: str | None = None,
    option_parser: OptionParser | None = None,
    config_parser: ConfigParser | None = None,
    config_section: str | None = None,
) -> str: ...
def validate_encoding_and_error_handler(
    setting: str,
    value: str | None = None,
    option_parser: OptionParser | None = None,
    config_parser: ConfigParser | None = None,
    config_section: str | None = None,
) -> str: ...
def validate_boolean(
    setting: str | bool,
    value: str | None = None,
    option_parser: OptionParser | None = None,
    config_parser: ConfigParser | None = None,
    config_section: str | None = None,
) -> bool: ...
def validate_ternary(
    setting: str | bool,
    value: str | None = None,
    option_parser: OptionParser | None = None,
    config_parser: ConfigParser | None = None,
    config_section: str | None = None,
) -> str | bool | None: ...
def validate_nonnegative_int(
    setting: str | int,
    value: str | None = None,
    option_parser: OptionParser | None = None,
    config_parser: ConfigParser | None = None,
    config_section: str | None = None,
) -> int: ...
def validate_threshold(
    setting: str | int,
    value: str | None = None,
    option_parser: OptionParser | None = None,
    config_parser: ConfigParser | None = None,
    config_section: str | None = None,
) -> int: ...
def validate_colon_separated_string_list(
    setting: str | list[str],
    value: str | None = None,
    option_parser: OptionParser | None = None,
    config_parser: ConfigParser | None = None,
    config_section: str | None = None,
) -> list[str]: ...
def validate_comma_separated_list(
    setting: str | list[str],
    value: str | None = None,
    option_parser: OptionParser | None = None,
    config_parser: ConfigParser | None = None,
    config_section: str | None = None,
) -> list[str]: ...
def validate_math_output(
    setting: str,
    value: str | None = None,
    option_parser: OptionParser | None = None,
    config_parser: ConfigParser | None = None,
    config_section: str | None = None,
) -> tuple[()] | tuple[str, str]: ...
def validate_url_trailing_slash(
    setting: str,
    value: str | None = None,
    option_parser: OptionParser | None = None,
    config_parser: ConfigParser | None = None,
    config_section: str | None = None,
) -> str: ...
def validate_dependency_file(
    setting: str | None,
    value: str | None = None,
    option_parser: OptionParser | None = None,
    config_parser: ConfigParser | None = None,
    config_section: str | None = None,
) -> DependencyList: ...
def validate_strip_class(
    setting: str,
    value: str | None = None,
    option_parser: OptionParser | None = None,
    config_parser: ConfigParser | None = None,
    config_section: str | None = None,
) -> list[str]: ...
def validate_smartquotes_locales(
    setting: str | list[str | tuple[str, str]],
    value: str | None = None,
    option_parser: OptionParser | None = None,
    config_parser: ConfigParser | None = None,
    config_section: str | None = None,
) -> list[tuple[str, Sequence[str]]]: ...
def make_paths_absolute(
    pathdict: dict[str, list[StrPath] | StrPath], keys: tuple[str], base_path: StrPath | None = None
) -> None: ...
@deprecated("The `frontend.make_one_path_absolute` will be removed in Docutils 2.0 or later.")
def make_one_path_absolute(base_path: StrPath, path: StrPath) -> str: ...
def filter_settings_spec(settings_spec, *exclude, **replace) -> tuple[Any, ...]: ...
@deprecated("The `frontend.Values` class will be removed in Docutils 2.0 or later.")
class Values(optparse.Values):
    record_dependencies: DependencyList
    def __init__(self, defaults: dict[str, Any] | None = None) -> None: ...
    def update(self, other_dict: Values | Mapping[str, Incomplete], option_parser: OptionParser) -> None: ...
    def copy(self) -> Values: ...
    def setdefault(self, name: str, default): ...

@deprecated("The `frontend.Option` class will be removed in Docutils 2.0 or later.")
class Option(optparse.Option):
    ATTRS: list[str]
    validator: _OptionValidator
    overrides: str | None
    def __init__(self, *args: str | None, **kwargs) -> None: ...

@deprecated(
    "The `frontend.OptionParser` class will be replaced by a subclass of `argparse.ArgumentParser` in Docutils 2.0 or later."
)
class OptionParser(optparse.OptionParser, SettingsSpec):
    standard_config_files: ClassVar[list[str]]
    threshold_choices: ClassVar[tuple[str, ...]]
    thresholds: ClassVar[dict[str, int]]
    booleans: ClassVar[dict[str, bool]]
    default_error_encoding: ClassVar[str]
    default_error_encoding_error_handler: ClassVar[str]
    config_section: ClassVar[str]
    version_template: ClassVar[str]
    details: str
    lists: dict[str, Literal[True]]
    config_files: list[str]
    relative_path_settings: ClassVar[tuple[str, ...]]
    version: str
    components: tuple[SettingsSpec, ...]
    def __init__(
        self,
        components: Iterable[SettingsSpec | type[SettingsSpec]] = (),
        defaults: Mapping[str, Any] | None = None,
        read_config_files: bool | None = False,
        *args,
        **kwargs,
    ) -> None: ...
    def populate_from_components(self, components: Iterable[SettingsSpec]) -> None: ...
    @classmethod
    def get_standard_config_files(cls) -> Sequence[StrPath]: ...
    def get_standard_config_settings(self) -> Values: ...
    def get_config_file_settings(self, config_file: str) -> dict[str, Incomplete]: ...
    def check_values(self, values: Values, args: list[str]) -> Values: ...  # type: ignore[override]
    def check_args(self, args: list[str]) -> tuple[str | None, str | None]: ...
    def get_default_values(self) -> Values: ...
    def get_option_by_dest(self, dest: str) -> Option: ...

class ConfigParser(RawConfigParser):
    old_settings: ClassVar[dict[str, tuple[str, str]]]
    old_warning: ClassVar[str]
    not_utf8_error: ClassVar[str]
    @overload  # type: ignore[override]
    def read(self, filenames: str | Sequence[str]) -> list[str]: ...
    @overload
    @deprecated("The `option_parser` parameter is deprecated and will be removed in Docutils 0.24.")
    def read(self, filenames: str | Sequence[str], option_parser: OptionParser | None) -> list[str]: ...
    def handle_old_config(self, filename: str) -> None: ...
    def validate_settings(self, filename: str, option_parser: OptionParser) -> None: ...
    def optionxform(self, optionstr: str) -> str: ...

class ConfigDeprecationWarning(FutureWarning): ...

def get_default_settings(*components: SettingsSpec) -> Values: ...



from typing import Literal

from .array import GeometryArray
from .base import GeoPandasBase
from .geodataframe import GeoDataFrame
from .geoseries import GeoSeries

def geom_equals(this: GeoPandasBase | GeometryArray, that: GeoPandasBase | GeometryArray) -> bool: ...
def geom_almost_equals(this: GeoPandasBase | GeometryArray, that: GeoPandasBase | GeometryArray) -> bool: ...
def assert_geoseries_equal(
    left: GeoSeries,
    right: GeoSeries,
    check_dtype: bool = True,
    check_index_type: bool = False,
    check_series_type: bool = True,
    check_less_precise: bool = False,
    check_geom_type: bool = False,
    check_crs: bool = True,
    normalize: bool = False,
) -> None: ...
def assert_geodataframe_equal(
    left: GeoDataFrame,
    right: GeoDataFrame,
    check_dtype: bool = True,
    check_index_type: bool | Literal["equiv"] = "equiv",
    check_column_type: bool | Literal["equiv"] = "equiv",
    check_frame_type: bool = True,
    check_like: bool = False,
    check_less_precise: bool = False,
    check_geom_type: bool = False,
    check_crs: bool = True,
    normalize: bool = False,
) -> None: ...



from collections.abc import Sequence

from pika.adapters.blocking_connection import ReturnedMessage

class AMQPError(Exception): ...
class AMQPConnectionError(AMQPError): ...
class ConnectionOpenAborted(AMQPConnectionError): ...
class StreamLostError(AMQPConnectionError): ...
class IncompatibleProtocolError(AMQPConnectionError): ...
class AuthenticationError(AMQPConnectionError): ...
class ProbableAuthenticationError(AMQPConnectionError): ...
class ProbableAccessDeniedError(AMQPConnectionError): ...
class NoFreeChannels(AMQPConnectionError): ...
class ConnectionWrongStateError(AMQPConnectionError): ...

class ConnectionClosed(AMQPConnectionError):
    def __init__(self, reply_code: int, reply_text: str) -> None: ...
    @property
    def reply_code(self) -> int: ...
    @property
    def reply_text(self) -> str: ...

class ConnectionClosedByBroker(ConnectionClosed): ...
class ConnectionClosedByClient(ConnectionClosed): ...
class ConnectionBlockedTimeout(AMQPConnectionError): ...
class AMQPHeartbeatTimeout(AMQPConnectionError): ...
class AMQPChannelError(AMQPError): ...
class ChannelWrongStateError(AMQPChannelError): ...

class ChannelClosed(AMQPChannelError):
    def __init__(self, reply_code: int, reply_text: str) -> None: ...
    @property
    def reply_code(self) -> int: ...
    @property
    def reply_text(self) -> str: ...

class ChannelClosedByBroker(ChannelClosed): ...
class ChannelClosedByClient(ChannelClosed): ...
class DuplicateConsumerTag(AMQPChannelError): ...
class ConsumerCancelled(AMQPChannelError): ...

class UnroutableError(AMQPChannelError):
    messages: Sequence[ReturnedMessage]
    def __init__(self, messages: Sequence[ReturnedMessage]) -> None: ...

class NackError(AMQPChannelError):
    messages: Sequence[ReturnedMessage]
    def __init__(self, messages: Sequence[ReturnedMessage]) -> None: ...

class InvalidChannelNumber(AMQPError): ...
class ProtocolSyntaxError(AMQPError): ...
class UnexpectedFrameError(ProtocolSyntaxError): ...
class ProtocolVersionMismatch(ProtocolSyntaxError): ...
class BodyTooLongError(ProtocolSyntaxError): ...
class InvalidFrameError(ProtocolSyntaxError): ...
class InvalidFieldTypeException(ProtocolSyntaxError): ...
class UnsupportedAMQPFieldException(ProtocolSyntaxError): ...
class MethodNotImplemented(AMQPError): ...
class ChannelError(Exception): ...
class ReentrancyError(Exception): ...
class ShortStringTooLong(AMQPError): ...
class DuplicateGetOkCallback(ChannelError): ...
